import React, { useEffect } from 'react';
import useNotificationService from '../notification/useNotificationService';
class AppRenderErrorBoundary extends React.Component {
    constructor(props) {
        super(props);
        this.state = { hasError: false };
    }
    static getDerivedStateFromError() {
        return { hasError: true };
    }
    componentDidCatch(error, errorInfo) {
        this.setState({
            hasError: true,
            error,
            errorInfo,
        });
    }
    render() {
        if (this.state.hasError) {
            const ErrorBoundaryComponent = this.props.ErrorBoundaryComponent;
            return (React.createElement(ErrorBoundaryComponent, { error: this.state.error, errorInfo: this.state.errorInfo, context: this.props.context }));
        }
        return this.props.children;
    }
}
const AppErrorBoundary = (props) => {
    const notificationService = useNotificationService();
    const ErrorBoundaryComponent = props.ErrorBoundaryComponent;
    useEffect(() => {
        const unhandledRejectionEventListener = (e) => {
            console.error(e);
            notificationService.error(e.reason);
        };
        if (window) {
            window.addEventListener('unhandledrejection', unhandledRejectionEventListener);
        }
        return () => {
            window.removeEventListener('unhandledrejection', unhandledRejectionEventListener);
        };
    }, [notificationService]);
    if (ErrorBoundaryComponent) {
        return React.createElement(AppRenderErrorBoundary, { ...props, ErrorBoundaryComponent: ErrorBoundaryComponent });
    }
    return React.createElement(React.Fragment, null, props.children);
};
export default AppErrorBoundary;
//# sourceMappingURL=AppErrorBoundary.js.map