import { Task } from 'redux-saga';
import { BasicError } from '../types/error';
import DefaultGlobalService from './GlobalService';
import { CacheEntry, CacheOptions } from './typings';
import { AnyState } from '../types/state';
export declare const cacheKey: (key: string) => string;
export declare const NO_EXPIRATION: number;
export default class CacheEntryService<T> extends DefaultGlobalService<AnyState> {
    delayLoading(key: string, delay_ms?: number): Generator<void | import("redux-saga/effects").CallEffect<true>, void, unknown>;
    /**
     * Save the object to cache in the redux store
     * Several metadata are atttached to the object (expireAt, loading, fetching)
     *
     * @param {object} cacheEntry : the object with metadata
     */
    setCache(key: string, cacheEntry?: CacheEntry<T>): void;
    setError(key: string, error?: BasicError): void;
    setLoading(key: string, loading: boolean, fetching: boolean): void;
    /**
     * Get the object from the server and save it in the cache (= redux store)
     *
     * @param {object} action :
     *    - onError: callback for any error
     *    - onSuccess: callback for any success
     * @param {object} context :
     *    - store: the redux store
     *    - router: the OnekiJS router of the application
     *    - settings: the full settings object passed to the application
     */
    fetch(url: string, options?: CacheOptions<T>, force?: boolean): Generator<void | import("redux-saga/effects").ForkEffect<Generator<void | import("redux-saga/effects").CallEffect<true>, void, unknown>> | Promise<any> | import("redux-saga/effects").CancelEffect | import("redux-saga/effects").CallEffect<unknown>, void, Task<any> & Response & T>;
}
