import 'reflect-metadata';
import DefaultBasicError from '../core/BasicError';
import { handler } from '../core/Service';
import { create, serviceClass } from '../types/service';
import { ID } from '../types/symbol';
import DefaultGlobalService from './GlobalService';
export default class Container {
    classRegistry;
    instanceRegistry;
    constructor() {
        this.classRegistry = {};
        this.instanceRegistry = {};
    }
    addServiceClass(ctor, force = true) {
        const label = ctor[ID];
        if (force || this.classRegistry[label] === undefined) {
            this.classRegistry[label] = ctor;
        }
    }
    getServiceClass(ctor) {
        return this.classRegistry[ctor[ID]] || ctor;
    }
    createService(ctor, context, initialState) {
        const instance = this.instanceRegistry[ctor[ID]];
        if (instance) {
            return instance;
        }
        const types = Reflect.getMetadata('design:paramtypes', ctor) || [];
        const args = types.map((type) => {
            if (!type[ID]) {
                throw new DefaultBasicError(`Cannot find a valid class for service ${type}`);
            }
            return this.instanceRegistry[type[ID]] || this.createService(type, context, context.store.getState());
        });
        const ActualClass = this.getServiceClass(ctor);
        const service = new ActualClass(...args);
        service.context = context;
        Object.getOwnPropertyNames(service).forEach((property) => {
            if (service[property] && service[property][serviceClass]) {
                const dependency = this.createService(service[property][serviceClass], context, context.store.getState());
                service[property] = dependency;
            }
        });
        service[create](initialState);
        const proxy = new Proxy(service, handler);
        if (service instanceof DefaultGlobalService) {
            this.instanceRegistry[ctor[ID]] = proxy;
        }
        return proxy;
    }
}
//# sourceMappingURL=Container.js.map