import { useCallback, useEffect } from 'react';
import useLazyRef from '../core/useLazyRef';
import CacheEntryService, { cacheKey } from './CacheEntryService';
import useGlobalProp from './useGlobalProp';
import useGlobalService from './useGlobalService';
const useCache = (url, options = {}) => {
    const optionsRef = useLazyRef(() => {
        return options;
    });
    const service = useGlobalService(CacheEntryService);
    const refresh = useCallback(() => {
        if (url) {
            service.fetch(url, optionsRef.current, true);
        }
    }, [url, service, optionsRef]);
    useEffect(() => {
        if (url) {
            service.fetch(url, optionsRef.current, false);
        }
    }, [url, optionsRef, service]);
    const result = useGlobalProp(cacheKey(url || '__NOT_DEFINED'), {
        loading: false,
        fetching: false,
        payload: options.defaultValue,
        ttl: options.ttl || 3600,
    });
    return [result.payload, result.loading || false, refresh];
};
export default useCache;
//# sourceMappingURL=useCache.js.map