import { useCallback, useEffect } from 'react';
import useLocalService from '../app/useLocalService';
import { useErrorCallback, useSuccessCallback } from '../app/utils';
import useNotificationService from '../notification/useNotificationService';
import LogoutService from './LogoutService';
const useLogout = (options = {}) => {
    const [state, service] = useLocalService(LogoutService, {
        loading: true,
    });
    const notificationService = useNotificationService();
    // we send errors to the notification service
    const defaultOnError = useCallback((error) => {
        notificationService.send({
            topic: 'logout-error',
            payload: error,
        });
    }, [notificationService]);
    const onError = useErrorCallback(options.onError) || defaultOnError;
    const onSuccess = useSuccessCallback(options.onSuccess);
    const callback = options.callback;
    const identity = options.identity ?? 'default';
    useEffect(() => {
        if (callback) {
            service.successLogout(onError, onSuccess, identity);
        }
        else {
            service.logout(onError, onSuccess, identity);
        }
    }, [service, onError, onSuccess, callback, identity]);
    return [state.error, state.loading || false];
};
// alias
export const useLogoutService = useLogout;
export default useLogout;
//# sourceMappingURL=useLogout.js.map