import { clone, toArray } from '../utils/object';
import { addFilter, formatFilter, formatSortBy, isSameSortBy, rootFilterId, visitFilter } from './utils';
export default class DefaultCollectionBroker {
    subscribers = {};
    filters;
    subscriberFilters = {};
    sortBys;
    subscriberSortBys = {};
    fields;
    subscriberFields = {};
    params;
    subscriberParams = {};
    currentSearch;
    subscriberSearch = {};
    currentSort;
    subscriberSort = {};
    data;
    subscriberData = {};
    url;
    subscriberUrl = {};
    initialLimit;
    initialOffset;
    constructor(dataSource, options) {
        if (Array.isArray(dataSource)) {
            this.data = dataSource;
        }
        else if (typeof dataSource === 'string') {
            this.url = dataSource;
        }
        this.filters = options.initialFilter;
        this.sortBys = options.initialSortBy;
        this.fields = options.initialFields;
        this.params = options.initialParams;
        this.currentSearch = options.initialSearch;
        this.currentSort = options.initialSort;
        this.initialLimit = options.initialLimit;
        this.initialOffset = options.initialOffset;
    }
    addFilter(filterOrCriteria, parentFilterId = rootFilterId, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberFilters).forEach((subscriberId) => {
                this._addFilter(filterOrCriteria, parentFilterId, subscriberId);
            });
            this._addFilter(filterOrCriteria, parentFilterId);
        }
        else {
            this._addFilter(filterOrCriteria, parentFilterId, subscriberId);
        }
        this._getSubscribers(subscriberId).forEach((s) => s.addFilter(filterOrCriteria, parentFilterId));
    }
    _addFilter(filterOrCriteria, parentFilterId = rootFilterId, subscriberId) {
        const query = this._getQuery(subscriberId);
        addFilter(query, filterOrCriteria, parentFilterId);
        if (subscriberId === undefined) {
            this.filters = query.filter;
        }
        else {
            this.subscriberFilters[subscriberId] = query.filter;
        }
    }
    addFilterCriteria(field, operator, value, not, id, parentFilterId, subscriberId) {
        this.addFilter({
            field,
            operator,
            value,
            not,
            id,
        }, parentFilterId, subscriberId);
    }
    addSortBy(sortBy, prepend, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberSortBys).forEach((subscriberId) => {
                this._addSortBy(sortBy, prepend, subscriberId);
            });
            this._addSortBy(sortBy, prepend);
        }
        else {
            this._addSortBy(sortBy, prepend, subscriberId);
        }
        this._getSubscribers(subscriberId).forEach((s) => s.addSortBy(sortBy, prepend));
    }
    _addSortBy(sortBy, prepend, subscriberId) {
        const sortBys = formatSortBy(this.getSortBy(subscriberId));
        if (sortBys) {
            if (subscriberId === undefined) {
                this.sortBys = sortBys.filter((s) => !isSameSortBy(sortBy, s));
            }
            else {
                this.subscriberSortBys[subscriberId] = sortBys.filter((s) => !isSameSortBy(sortBy, s));
            }
        }
        const currentSortBy = (formatSortBy(this.getSortBy(subscriberId)) || []).slice(0);
        if (prepend) {
            currentSortBy.unshift(sortBy);
        }
        else {
            currentSortBy.push(sortBy);
        }
        if (subscriberId === undefined) {
            this.sortBys = currentSortBy;
        }
        else {
            this.subscriberSortBys[subscriberId] = currentSortBy;
        }
    }
    addSubscriber(subscriberId, subscriber) {
        if (!this.subscribers[subscriberId]) {
            this.subscribers[subscriberId] = subscriber;
            const initialData = this.getData(subscriberId);
            const initialUrl = this.getUrl(subscriberId);
            const initialQuery = this.getInitialQuery(subscriberId);
            subscriber.onSubscribe(initialData, initialUrl, initialQuery);
        }
    }
    clearFields(subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberFields).forEach((subscriberId) => {
                this.subscriberFields[subscriberId] = [];
            });
            this.fields = [];
        }
        else {
            this.subscriberFields[subscriberId] = [];
        }
        this._getSubscribers(subscriberId).forEach((s) => s.clearFields());
    }
    clearFilter(subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberFilters).forEach((subscriberId) => {
                this.subscriberFilters[subscriberId] = [];
            });
            this.filters = [];
        }
        else {
            this.subscriberFilters[subscriberId] = [];
        }
        this._getSubscribers(subscriberId).forEach((s) => s.clearFilter());
    }
    clearParam(key, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberParams).forEach((subscriberId) => {
                const params = this.subscriberParams[subscriberId];
                if (params !== undefined) {
                    delete params[key];
                }
            });
            if (this.params !== undefined) {
                delete this.params[key];
            }
        }
        else {
            const params = this.subscriberParams[subscriberId];
            if (params !== undefined) {
                delete params[key];
            }
        }
        this._getSubscribers(subscriberId).forEach((s) => s.clearParam(key));
    }
    clearParams(subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberParams).forEach((subscriberId) => {
                this.subscriberParams[subscriberId] = {};
            });
            this.params = {};
        }
        else {
            this.subscriberParams[subscriberId] = {};
        }
        this._getSubscribers(subscriberId).forEach((s) => s.clearParams());
    }
    clearSearch(subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberSearch).forEach((subscriberId) => {
                this.subscriberSearch[subscriberId] = undefined;
            });
            this.currentSearch = undefined;
        }
        else {
            this.subscriberSearch[subscriberId] = undefined;
        }
        this._getSubscribers(subscriberId).forEach((s) => s.clearSearch());
    }
    clearSort(subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberSort).forEach((subscriberId) => {
                this.subscriberSort[subscriberId] = undefined;
            });
            this.currentSort = undefined;
        }
        else {
            this.subscriberSort[subscriberId] = undefined;
        }
        this._getSubscribers(subscriberId).forEach((s) => s.clearSort());
    }
    clearSortBy(subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberSortBys).forEach((subscriberId) => {
                this.subscriberSortBys[subscriberId] = undefined;
            });
            this.sortBys = [];
        }
        else {
            this.subscriberSortBys[subscriberId] = [];
        }
        this._getSubscribers(subscriberId).forEach((s) => s.clearSortBy());
    }
    filter(filter, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberFilters).forEach((subscriberId) => {
                this.subscriberFilters[subscriberId] = formatFilter(filter);
            });
            this.filters = formatFilter(filter);
        }
        else {
            this.subscriberFilters[subscriberId] = formatFilter(filter);
        }
        this._getSubscribers(subscriberId).forEach((s) => s.filter(filter));
    }
    getData(subscriberId) {
        return subscriberId && this.subscriberData[subscriberId] ? this.subscriberData[subscriberId] : this.data;
    }
    getFields(subscriberId) {
        return subscriberId && this.subscriberFields[subscriberId] ? this.subscriberFields[subscriberId] : this.fields;
    }
    getFilter(subscriberId) {
        return subscriberId && this.subscriberFilters[subscriberId] ? this.subscriberFilters[subscriberId] : this.filters;
    }
    getInitialQuery(subscriberId) {
        return {
            filter: formatFilter(this.getFilter(subscriberId)),
            sortBy: formatSortBy(this.getSortBy(subscriberId)),
            fields: this.getFields(subscriberId),
            search: this.getSearch(subscriberId),
            sort: this.getSort(subscriberId),
            params: this.getParams(subscriberId),
            limit: this.initialLimit,
            offset: this.initialOffset,
        };
    }
    getInitialDataSource(subscriberId) {
        if (Array.isArray(this.getData(subscriberId))) {
            return this.data;
        }
        return this.getUrl(subscriberId);
    }
    getParams(subscriberId) {
        return subscriberId && this.subscriberParams[subscriberId] ? this.subscriberParams[subscriberId] : this.params;
    }
    getSearch(subscriberId) {
        return subscriberId && this.subscriberSearch[subscriberId]
            ? this.subscriberSearch[subscriberId]
            : this.currentSearch;
    }
    getSort(subscriberId) {
        return subscriberId && this.subscriberSort[subscriberId] ? this.subscriberSort[subscriberId] : this.currentSort;
    }
    getSortBy(subscriberId) {
        return subscriberId && this.subscriberSortBys[subscriberId] ? this.subscriberSortBys[subscriberId] : this.sortBys;
    }
    getUrl(subscriberId) {
        return subscriberId && this.subscriberUrl[subscriberId] ? this.subscriberUrl[subscriberId] : this.url;
    }
    refresh(query, subscriberId) {
        this._getSubscribers(subscriberId).forEach((s) => s.refresh(query));
    }
    removeFilter(id, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberFilters).forEach((subscriberId) => {
                this._removeFilter(id, subscriberId);
            });
            this._removeFilter(id);
        }
        else {
            this._removeFilter(id, subscriberId);
        }
        this._getSubscribers(subscriberId).forEach((s) => s.removeFilter(id));
    }
    _removeFilter(id, subscriberId) {
        const filter = clone(formatFilter(this.getFilter(subscriberId)));
        if (filter) {
            visitFilter(filter, (filter) => {
                for (const i in filter.criterias) {
                    if (filter.criterias[i].id === id) {
                        filter.criterias.splice(parseInt(i), 1);
                        return true;
                    }
                }
                return false;
            });
            if (subscriberId === undefined) {
                this.filters = filter;
            }
            else {
                this.subscriberFilters[subscriberId] = filter;
            }
        }
    }
    removeSortBy(id, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberSortBys).forEach((subscriberId) => {
                this._removeSortBy(id, subscriberId);
            });
            this._removeSortBy(id);
        }
        else {
            this._removeSortBy(id, subscriberId);
        }
        this._getSubscribers(subscriberId).forEach((s) => s.removeSortBy(id));
    }
    _removeSortBy(id, subscriberId) {
        const sortBy = formatSortBy(this.getSortBy(subscriberId));
        if (sortBy) {
            if (subscriberId === undefined) {
                this.sortBys = sortBy.filter((sort) => sort.id !== id);
            }
            else {
                this.subscriberSortBys[subscriberId] = sortBy.filter((sort) => sort.id !== id);
            }
        }
    }
    removeSubscriber(id) {
        delete this.subscribers[id];
    }
    search(search, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberSearch).forEach((subscriberId) => {
                this.subscriberSearch[subscriberId] = search;
            });
            this.currentSearch = search;
        }
        else {
            this.subscriberSearch[subscriberId] = search;
        }
        this._getSubscribers(subscriberId).forEach((s) => s.search(search));
    }
    setData(data, query, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberData).forEach((subscriberId) => {
                this.subscriberData[subscriberId] = data;
            });
            this.data = data;
        }
        else {
            this.subscriberData[subscriberId] = data;
        }
        if (query) {
            this._setQuery(query, subscriberId);
        }
        this._getSubscribers(subscriberId).forEach((s) => s.setData(data, query));
    }
    setFields(fields, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberFields).forEach((subscriberId) => {
                this.subscriberFields[subscriberId] = fields;
            });
            this.fields = fields;
        }
        else {
            this.subscriberFields[subscriberId] = fields;
        }
        this._getSubscribers(subscriberId).forEach((s) => s.setFields(fields));
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    setParam(key, value, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberParams).forEach((subscriberId) => {
                if (this.subscriberParams[subscriberId] === undefined) {
                    this.subscriberParams[subscriberId] = this.params || {};
                }
                this.subscriberParams[subscriberId][key] = value;
            });
            if (this.params === undefined) {
                this.params = {};
            }
            this.params[key] = value;
        }
        else {
            if (this.subscriberParams[subscriberId] === undefined) {
                this.subscriberParams[subscriberId] = this.params || {};
            }
            this.subscriberParams[subscriberId][key] = value;
        }
        this._getSubscribers(subscriberId).forEach((s) => s.setParam(key, value));
    }
    setParams(params, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberParams).forEach((subscriberId) => {
                this.subscriberParams[subscriberId] = params;
            });
            this.params = params;
        }
        else {
            this.subscriberParams[subscriberId] = params;
        }
        this._getSubscribers(subscriberId).forEach((s) => s.setParams(params));
    }
    setUrl(url, query, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberUrl).forEach((subscriberId) => {
                this.subscriberUrl[subscriberId] = url;
            });
            this.url = url;
        }
        else {
            this.subscriberUrl[subscriberId] = url;
        }
        if (query) {
            this._setQuery(query);
        }
        this._getSubscribers(subscriberId).forEach((s) => s.setUrl(url, query));
    }
    sort(dir, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberSort).forEach((subscriberId) => {
                this.subscriberSort[subscriberId] = dir;
            });
            this.currentSort = dir;
        }
        else {
            this.subscriberSort[subscriberId] = dir;
        }
        this._getSubscribers(subscriberId).forEach((s) => s.sort(dir));
    }
    sortBy(sortBy, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscriberSortBys).forEach((subscriberId) => {
                this.subscriberSortBys[subscriberId] = formatSortBy(sortBy);
            });
            this.sortBys = formatSortBy(sortBy);
        }
        else {
            this.subscriberSortBys[subscriberId] = formatSortBy(sortBy);
        }
        this._getSubscribers(subscriberId).forEach((s) => s.sortBy(sortBy));
    }
    _getSubscribers(subscriberId) {
        return subscriberId !== undefined ? toArray(this.subscribers[subscriberId]) : Object.values(this.subscribers);
    }
    _getQuery(subscriberId) {
        return {
            filter: formatFilter(this.getFilter(subscriberId)),
            sortBy: formatSortBy(this.getSortBy(subscriberId)),
            fields: this.getFields(subscriberId),
            search: this.getSearch(subscriberId),
            sort: this.getSort(subscriberId),
            params: this.getParams(subscriberId),
        };
    }
    _setQuery(query, subscriberId) {
        if (subscriberId === undefined) {
            Object.keys(this.subscribers).forEach((subscriberId) => {
                this.subscriberFilters[subscriberId] = query.filter;
                this.subscriberSortBys[subscriberId] = query.sortBy;
                this.subscriberFields[subscriberId] = query.fields;
                this.subscriberSearch[subscriberId] = query.search;
                this.subscriberSort[subscriberId] = query.sort;
                this.subscriberParams[subscriberId] = query.params;
            });
            this.filters = query.filter;
            this.sortBys = query.sortBy;
            this.fields = query.fields;
            this.currentSearch = query.search;
            this.currentSort = query.sort;
            this.params = query.params;
        }
        else {
            this.subscriberFilters[subscriberId] = query.filter;
            this.subscriberSortBys[subscriberId] = query.sortBy;
            this.subscriberFields[subscriberId] = query.fields;
            this.subscriberSearch[subscriberId] = query.search;
            this.subscriberSort[subscriberId] = query.sort;
            this.subscriberParams[subscriberId] = query.params;
        }
    }
}
//# sourceMappingURL=CollectionBroker.js.map