var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r;
import { __decorate, __metadata } from "tslib";
import { cancel, delay, fork } from 'redux-saga/effects';
import DefaultBasicError from '../core/BasicError';
import DefaultService from '../core/Service';
import { reducer, saga } from '../core/annotations';
import { asyncHttp } from '../core/xhr';
import LocalRouter from '../router/LocalRouter';
import { HttpMethod } from '../types/fetch';
import { SagaEffect } from '../types/saga';
import { dispatch, types } from '../types/service';
import { clone, ensureFieldValue, get, isNull, set, toArray, toPayload } from '../utils/object';
import { urlBuilder } from '../utils/router';
import { generateUniqueId } from '../utils/string';
import { LoadingStatus, } from './typings';
import { addFilter as aFilter, defaultComparator, defaultQueryEngine, defaultSerializer, formatFilter, formatSortBy, getDefaultCollectionStatus, isCollectionReady, isQueryFilterCriteria, isQuerySortByField, isQuerySortByMultiFields, isSameQuery, isSameSortBy, parseQuery, rootFilterId, shouldResetData, toQuerySortBy, urlSerializer, visitFilter, } from './utils';
export default class CollectionService extends DefaultService {
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    _initialState = null;
    _cache = {};
    _idIndex = {};
    _uidIndex = {};
    _positionIndex = {};
    _db;
    _unregisterRouterListener;
    _currentQuery;
    _initialQuery;
    _refreshing;
    _noCache;
    _noLoading;
    _resetData = false;
    refreshTask;
    followTask;
    scrollToIndex;
    scrollToOffset;
    init() {
        this._initialState = this.state;
        if (this.state.filter) {
            this.state.filter = formatFilter(this.state.filter);
        }
        if (this.state.sortBy) {
            this.state.sortBy = toQuerySortBy(this.state.sortBy);
        }
        if (!this.state.router) {
            this.state.router = new LocalRouter();
        }
        // listen on location change and adapt filters, sort, ... with these values
        this._unregisterRouterListener = this.state.router.listen((location) => this._onLocationChange(location));
        // retrieve params from URL and initiate filter, sort ... with these values
        this._setQuery(this._parseLocation(this.state.router.location), false);
        this.initDb(this.state.dataSource);
        if (this.state.local === undefined) {
            this.state.local =
                this.state.dataSource === undefined || Array.isArray(this.state.dataSource) || this.state.fetchOnce === true;
        }
        if (this.state.local && this.state.dataSource !== undefined) {
            this.refresh();
        }
        this._initialQuery = this.getQuery();
    }
    destroy() {
        if (this._unregisterRouterListener) {
            this._unregisterRouterListener();
        }
    }
    initDb(dataSource) {
        if (Array.isArray(dataSource)) {
            this.state.status = LoadingStatus.Loaded;
            this._db = [];
            dataSource.map((entry, index) => this._adapt(entry, { position: index }));
        }
    }
    adapt(data) {
        return this._adapt(data);
    }
    addActive(by, target) {
        return this._addByMeta('active', by, target);
    }
    addDisabled(by, target) {
        return this._addByMeta('disabled', by, target);
    }
    addFilter(filterOrCriteria, parentFilterId = rootFilterId) {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = this.getQuery();
        this._addFilter(query, filterOrCriteria, parentFilterId);
        this.refresh(query);
    }
    addFilterCriteria(field, operator, value, not, id, parentFilterId) {
        this.addFilter({
            field,
            operator,
            value,
            not,
            id,
        }, parentFilterId);
    }
    addHighlighted(by, target) {
        return this._addByMeta('highlighted', by, target);
    }
    addSelected(by, target) {
        return this._addByMeta('selected', by, target);
    }
    addSortBy(sortBy, prepend) {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = this.getQuery();
        this._addSortBy(query, sortBy, prepend);
        this.refresh(query);
    }
    asService() {
        return this;
    }
    clearFields() {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = this.getQuery();
        this._clearFields(query);
        this.refresh(query);
    }
    clearFilter() {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = this.getQuery();
        this._clearFilter(query);
        this.refresh(query);
    }
    clearLimit() {
        this._setLoading({ offset: this.state.offset, resetData: false });
        const query = this.getQuery();
        this._clearLimit(query);
        this.refresh(query);
    }
    clearOffset() {
        const query = this.getQuery();
        this._setLoading({ limit: this.state.limit, resetData: false });
        this._clearOffset(query);
        this.refresh(query);
    }
    clearParam(key) {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = this.getQuery();
        this._clearParam(query, key);
        this.refresh(query);
    }
    clearParams() {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = this.getQuery();
        this._clearParams(query);
        this.refresh(query);
    }
    clearSearch() {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = this.getQuery();
        this._clearSearch(query);
        this.refresh(query);
    }
    clearSort() {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = this.getQuery();
        this._clearSort(query);
        this.refresh(query);
    }
    clearSortBy() {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = this.getQuery();
        this._clearSortBy(query);
        this.refresh(query);
    }
    filter(filter) {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = this.getQuery();
        this._setFilter(query, filter);
        this.refresh(query);
    }
    get data() {
        const items = this.state.items;
        if (items !== undefined) {
            const result = items.reduce((accumulator, item) => {
                if (item?.data !== undefined) {
                    accumulator.push(item.data);
                }
                return accumulator;
            }, []);
            return result;
        }
        return undefined;
    }
    get dataSource() {
        return this.state.dataSource;
    }
    get hasMore() {
        return this.state.hasMore || false;
    }
    get items() {
        return this.state.items || [];
    }
    get index() {
        return this._idIndex;
    }
    get status() {
        const defaultStatus = getDefaultCollectionStatus(this._db || this.state.url, this.state.brokerable, this.state.brokered, this.state.fetchOnce);
        return this.state.status || defaultStatus;
    }
    get total() {
        return this.state.local ? this.state.items?.length : this.state.total;
    }
    get url() {
        return this.state.url || '';
    }
    getDb() {
        return this._db || [];
    }
    getFields() {
        return this.state.fields;
    }
    getFilter() {
        return formatFilter(this.state.filter);
    }
    getFilterById(id) {
        let result = undefined;
        const filter = formatFilter(this.state.filter);
        if (filter !== undefined) {
            visitFilter(filter, (filter) => {
                if (filter.id === id) {
                    result = filter;
                    return true;
                }
                for (const filterOrCriteria of filter.criterias) {
                    if (isQueryFilterCriteria(filterOrCriteria) && filterOrCriteria.id === id) {
                        result = filterOrCriteria;
                        return true;
                    }
                }
                return false;
            });
        }
        return result;
    }
    getItem(uid) {
        return this._uidIndex[uid];
    }
    getOffset() {
        return this.state.offset;
    }
    getParam(key) {
        return get(this.state.params, key);
    }
    getParams() {
        return this.state.params;
    }
    getQuery() {
        return {
            filter: this.getFilter(),
            sortBy: this.getSortBy(),
            offset: this.getOffset(),
            limit: this.getLimit(),
            fields: this.getFields(),
            search: this.getSearch(),
            sort: this.getSort(),
            params: this.getParams(),
        };
    }
    getSearch() {
        return this.state.search;
    }
    getLimit() {
        return this.state.limit;
    }
    getSort() {
        return this.state.sort;
    }
    getSortBy() {
        let currentSortBy = get(this.state, 'sortBy');
        if (currentSortBy === undefined) {
            currentSortBy = [];
        }
        else if (typeof currentSortBy === 'string') {
            currentSortBy = [{ field: currentSortBy }];
        }
        else if (!Array.isArray(currentSortBy)) {
            currentSortBy = [currentSortBy];
        }
        return currentSortBy;
    }
    getSortByField(field) {
        const sorts = this.getSortBy();
        if (sorts) {
            for (const sort of sorts) {
                if (isQuerySortByField(sort)) {
                    if (sort.field === field) {
                        return sort;
                    }
                }
                else if (isQuerySortByMultiFields(sort)) {
                    const field = sort.fields.find((f) => (typeof f === 'string' ? f === field : f.name === field));
                    return {
                        id: sort.id,
                        dir: sort.dir,
                        comparator: typeof field === 'string' ? undefined : field.comparator,
                        field: typeof field === 'string' ? field : field.name,
                    };
                }
            }
        }
        return undefined;
    }
    getSortById(id) {
        const sorts = this.getSortBy();
        if (sorts) {
            return sorts.find((sort) => sort.id === id);
        }
        return undefined;
    }
    isFiltered() {
        return this._isFiltered(this.getQuery());
    }
    load(limit, offset, replace = false) {
        limit = limit ?? this.getLimit();
        offset = offset ?? this.getOffset();
        if (!this.state.local) {
            const resetData = this.state.items ? false : true;
            this._setLoading({ limit, offset, resetData });
            this.refresh(undefined, !replace);
        }
    }
    onSubscribe(initialData, initialUrl, initialQuery) {
        this.state.brokered = true;
        if (Array.isArray(initialData)) {
            this.setData(initialData, initialQuery);
        }
        else if (initialUrl !== undefined) {
            this.setUrl(initialUrl, initialQuery);
        }
        else {
            this.query(initialQuery);
        }
    }
    query(query) {
        let resetData = false;
        if (['filter', 'search', 'sort', 'sortBy', 'fields'].filter((value) => Object.keys(query).includes(value)).length > 0) {
            resetData = true;
        }
        this._setLoading({ limit: query.limit, offset: query.offset, resetData });
        this._setQuery(query, resetData);
        this.refresh();
    }
    reload() {
        const query = Object.assign(this.getQuery(), {
            limit: this._initialQuery?.limit,
            offset: 0,
        });
        this.state.items = undefined;
        this.state.status = LoadingStatus.NotInitialized;
        this._setLoading({ limit: query.limit, offset: query.offset, resetData: true });
        this.refresh(query, false, true, false);
    }
    refresh(query, push, noCache, noLoading) {
        push = push ?? (query ? true : false);
        noCache = noCache ?? this._noCache ?? (query ? false : true);
        noLoading = noLoading ?? this._noLoading ?? false;
        if (query) {
            this._setQuery(query);
        }
        else {
            query = this.getQuery();
        }
        if (isCollectionReady(this)) {
            const path = this.state.router.location.pathname;
            if (noCache) {
                this._setParam(query, 'noCache', true);
            }
            else {
                this._clearParam(query, 'noCache');
            }
            if (noLoading) {
                this._setParam(query, 'noLoading', true);
            }
            else {
                this._clearParam(query, 'noLoading');
            }
            const method = push ? 'push' : 'replace';
            this._refreshing = true;
            this.state.router[method](urlBuilder(path, {}, urlSerializer(query)));
        }
    }
    removeActive(by, target) {
        return this._removeByMeta('active', by, target);
    }
    removeDisabled(by, target) {
        return this._removeByMeta('disabled', by, target);
    }
    removeFilter(filterId) {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = this.getQuery();
        this._removeFilter(query, filterId);
        this.refresh(query);
    }
    removeHighlighted(by, target) {
        return this._removeByMeta('highlighted', by, target);
    }
    removeSelected(by, target) {
        return this._removeByMeta('selected', by, target);
    }
    removeSortBy(id) {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = this.getQuery();
        this._removeSortById(query, id);
        this.refresh(query);
    }
    reset() {
        this.state = this._initialState;
        this._setLoading({ limit: this.state.limit, offset: this.state.offset });
        this.refresh();
    }
    search(search) {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = Object.assign({}, this.getQuery());
        this._setSearch(query, search);
        this.refresh(query);
    }
    serializeQuery(query) {
        const serializer = this.state.serializer || defaultSerializer;
        return serializer(query);
    }
    setActive(by, target) {
        return this._setByMeta('active', by, target);
    }
    setData(data, query) {
        if (!this.state.local) {
            throw new DefaultBasicError('Call to unsupported method setData of a remote collection');
        }
        this._cache = {};
        // this.idIndex = {};
        // this.uidIndex = {};
        this._positionIndex = {};
        this.state.items = undefined;
        if (query === undefined) {
            query = this.getQuery();
        }
        this._clearOffset(query);
        this.initDb(data);
        this.refresh(query);
    }
    setDisabled(by, target) {
        return this._setByMeta('disabled', by, target);
    }
    setFields(fields) {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = Object.assign({}, this.getQuery());
        this._setFields(query, fields);
        this.refresh(query);
    }
    setHighlighted(by, target) {
        return this._setByMeta('highlighted', by, target);
    }
    setMeta(by, target, key, value) {
        let items = [];
        target = toArray(target);
        switch (by) {
            case 'id':
                items = target.map((t) => this._idIndex[String(t)]).filter((t) => t !== undefined);
                break;
            case 'uid':
                items = target.map((t) => this._uidIndex[String(t)]).filter((t) => t !== undefined);
                break;
            case 'item':
                items = target.map((t) => this._uidIndex[t.uid]).filter((t) => t !== undefined);
                break;
            case 'value':
                items = target.map((t) => this.adapt(t));
                break;
        }
        const result = [];
        items.forEach((item) => {
            item = Object.assign({}, item, { [key]: value });
            this._indexItem(item);
            if (this.state.items) {
                const stateItem = this.state.items.find((stateItem) => stateItem && item.uid === stateItem.uid);
                if (stateItem) {
                    stateItem[key] = value;
                }
            }
            result.push(item);
        });
        return result;
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    setParam(key, value) {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = Object.assign({}, this.getQuery());
        this._setParam(query, key, value);
        this.refresh(query);
    }
    setParams(params) {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = Object.assign({}, this.getQuery());
        this._setParams(query, params);
        this.refresh(query);
    }
    setSelected(by, target) {
        const result = this._setByMeta('selected', by, target);
        return result;
    }
    setStatus(status) {
        this.state.status = status;
    }
    setUrl(url, query) {
        this._cache = {};
        this.state.dataSource = url;
        this.state.url = url;
        this.state.items = undefined;
        this.state.local = this.state.fetchOnce || false;
        this.state.status = LoadingStatus.NotInitialized;
        if (query === undefined) {
            query = this.getQuery();
        }
        else {
            this._setQuery(query);
        }
        const nextQuery = clone(query);
        this._clearOffset(nextQuery);
        if (this.state.fetchOnce) {
            this.callSaga('_fetchOnce');
        }
        else {
            this[dispatch]({
                type: this[types]._fetch.actionType,
                payload: toPayload([nextQuery, true]),
            });
        }
    }
    sort(dir) {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = Object.assign({}, this.getQuery());
        this._setSort(query, dir);
        this.refresh(query);
    }
    sortBy(sortBy) {
        this._setLoading({ limit: this.state.limit, offset: 0 });
        const query = Object.assign({}, this.getQuery());
        this._setSortBy(query, sortBy);
        this.refresh(query);
    }
    *startAutoRefresh(interval) {
        yield this.stopAutoRefresh();
        this.refreshTask = yield fork([this, this._autoRefresh], interval);
    }
    *startFollow(interval) {
        yield this._disableDelayLoading();
        yield this.stopFollow();
        this._noCache = true;
        this._noLoading = true;
        this.followTask = yield fork([this, this._follow], interval);
    }
    *stopAutoRefresh() {
        if (this.refreshTask) {
            yield this.refreshTask.cancel();
            this.refreshTask = undefined;
        }
    }
    *stopFollow() {
        if (this.followTask) {
            this._noCache = undefined;
            this._noLoading = undefined;
            yield this.followTask.cancel();
            this.followTask = undefined;
        }
    }
    _adapt(data, context) {
        const adaptee = this.state.adapter === undefined || data === undefined || data === null ? {} : this.state.adapter(data);
        const item = this._buildItem(data, adaptee, context);
        if (context === undefined || !context.doNotIndex) {
            this._indexItem(item, context);
        }
        return item;
    }
    _addByMeta(field, by, target) {
        const items = this.setMeta(by, target, field, true);
        const arr = this.state[field] || [];
        items.forEach((item) => {
            if (!arr.includes(item.uid)) {
                arr.push(item.uid);
            }
        });
        // this.state[field] = [...new Set((this.state[field] || []).concat(items.map((item) => item.uid)))];
        this.state[field] = arr;
        return items;
    }
    _addFilter(query, filterOrCriteria, parentFilterId = rootFilterId) {
        return aFilter(query, filterOrCriteria, parentFilterId);
    }
    _addSortBy(query, sortBy, prepend = true) {
        this._removeSortBy(query, sortBy);
        const currentSortBy = (formatSortBy(get(query, 'sortBy'), get(this.state, 'sortBy')) || []).slice(0);
        if (prepend) {
            currentSortBy.unshift(sortBy);
        }
        else {
            currentSortBy.push(sortBy);
        }
        query.sortBy = currentSortBy;
    }
    *_autoRefresh(interval) {
        while (true) {
            yield delay(interval);
            yield this.refresh(undefined, false, true, true);
        }
    }
    _clearFields(query) {
        query.fields = undefined;
    }
    _clearFilter(query) {
        query.filter = undefined;
    }
    _clearOffset(query) {
        query.offset = undefined;
    }
    _clearParam(query, key) {
        if (query.params) {
            delete query.params[key];
        }
    }
    _clearParams(query) {
        query.params = undefined;
    }
    _clearSearch(query) {
        query.search = undefined;
    }
    _clearLimit(query) {
        query.limit = undefined;
    }
    _clearSortBy(query) {
        query.sortBy = undefined;
    }
    _clearSort(query) {
        query.sort = undefined;
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    *_delayLoading(delay_ms, limit, offset, resetData) {
        if (typeof limit === 'string') {
            limit = parseInt(limit);
        }
        if (typeof offset === 'string') {
            offset = parseInt(offset);
        }
        yield delay(delay_ms);
        yield this._setLoading({
            status: LoadingStatus.Loading,
            limit,
            offset,
            resetLimit: false,
            resetData,
        });
    }
    _disableDelayLoading() {
        if (this.state.fetchOptions?.delayLoading) {
            this.state.fetchOptions.delayLoading = 0;
        }
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    *_fetchOnce() {
        if (typeof this.state.dataSource === 'string' && this.state.fetchOnce && this.status !== LoadingStatus.NotReady) {
            try {
                let result = [];
                if (this.status !== LoadingStatus.Loaded) {
                    yield this.setStatus(LoadingStatus.Loading);
                    result = yield this._executeFetch(Object.assign({}, this._initialQuery, { limit: undefined, offset: undefined }), this.state.fetchOptions, false);
                }
                // check if the select was not converted to a local datasource during the fetch
                if (this.status !== LoadingStatus.Loaded) {
                    yield this.setStatus(LoadingStatus.Loaded);
                    if (Array.isArray(result)) {
                        yield this.setData(result);
                    }
                    else {
                        yield this.setData(result[this.state.dataKey]);
                    }
                }
            }
            catch (e) {
                if (process.env.NODE_ENV === 'development') {
                    console.error('Fetch error', e);
                }
                if (this.state.fetchOptions?.onFetchError) {
                    yield this.state.fetchOptions.onFetchError(DefaultBasicError.of(e));
                }
                else {
                    throw e;
                }
            }
        }
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    *_fetch(query, resetData) {
        const options = this.state.fetchOptions || {};
        const { onFetchError } = options;
        try {
            const result = yield this._executeFetch(query, options, resetData);
            yield this._fetchSuccess(result, resetData, query); // to update the store and trigger a re-render.
        }
        catch (e) {
            if (process.env.NODE_ENV === 'development') {
                console.error('Fetch error', e);
            }
            yield this._fetchError(e, query);
            if (onFetchError) {
                yield onFetchError(DefaultBasicError.of(e));
            }
            else {
                throw e;
            }
        }
    }
    *_follow(interval) {
        while (true) {
            yield delay(interval);
            const query = this.getQuery();
            query.offset = this.state.total || 0;
            yield this.refresh(query, false, true, true);
        }
    }
    *_executeFetch(query, options = {}, resetData) {
        if (!this.url) {
            return [];
        }
        let loadingTask = null;
        try {
            const noCache = get(query, 'params.noCache', false);
            const noLoading = get(query, 'params.noLoading', false);
            const oQuery = this.serializeQuery(query);
            const sQuery = Object.keys(oQuery)
                .map((k) => `${k}=${oQuery[k]}`)
                .join('&');
            let result;
            if (this._cache[sQuery] && !noCache) {
                result = this._cache[sQuery];
            }
            else {
                if (options.delayLoading && !noLoading) {
                    loadingTask = yield fork([this, this._delayLoading], options.delayLoading, query.limit, query.offset, resetData);
                }
                const fetcher = options.fetcher || asyncHttp;
                const method = this.state.method ?? HttpMethod.Get;
                const body = this.state.method === HttpMethod.Get ? undefined : Object.assign({}, query);
                const fetchOptions = method === HttpMethod.Get ? Object.assign({}, options, { query: oQuery }) : options;
                result = yield fetcher(this.url, method, body, fetchOptions);
                if (this.state.fetchOptions?.onFetchSuccess) {
                    const formattedResult = yield this.state.fetchOptions.onFetchSuccess(result);
                    if (formattedResult !== undefined) {
                        result = formattedResult;
                    }
                }
                this._cache[sQuery] = result;
                if (loadingTask !== null) {
                    yield cancel(loadingTask);
                }
            }
            return result;
        }
        catch (e) {
            if (loadingTask !== null) {
                yield cancel(loadingTask);
            }
            throw e;
        }
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    _fetchError(e, query) {
        this.state.error = e;
        this._setLoading({
            resetLimit: false,
            resetData: false,
            limit: query.limit,
            offset: query.offset,
            status: LoadingStatus.Error,
        });
        this.state.status = LoadingStatus.Error;
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    _fetchSuccess(result, resetData, query) {
        const limit = query.limit;
        const offset = query.offset || 0;
        const currentQuery = this.getQuery();
        const data = Array.isArray(result) ? result : result[this.state.dataKey];
        const total = Array.isArray(result) ? undefined : result[this.state.totalKey];
        let hasMore = true;
        if (Array.isArray(result)) {
            if (!limit || result.length < limit) {
                hasMore = false;
            }
        }
        else {
            if (!limit) {
                hasMore = false;
            }
            else if (result[this.state.hasMoreKey] === true) {
                hasMore = true;
            }
            else if (Object.keys(result).includes(this.state.totalKey)) {
                hasMore = total > limit + offset;
            }
            else {
                hasMore = data.length >= limit;
            }
        }
        // check if current query is still equals to the query used for fetching
        const q1 = Object.assign({}, query);
        const q2 = Object.assign({}, currentQuery);
        delete q1.limit;
        delete q1.offset;
        delete q2.limit;
        delete q2.offset;
        let same = isSameQuery(q1, q2);
        this.state.error = undefined;
        this.state.total = undefined;
        if (same) {
            // we have marked old items as loading. We need to mark them as loaded (even if they are no more in the list)
            this._setLoading({
                resetLimit: false,
                resetData,
                limit: query.limit,
                offset: query.offset,
                status: LoadingStatus.Loaded,
            });
            // update metadata
            let itemResult = data.map((itemData) => {
                const item = this.adapt(itemData);
                item.loadingStatus = LoadingStatus.Loaded;
                return Object.assign({}, item);
            });
            if (this.state.onQuerySuccess) {
                itemResult = this.state.onQuerySuccess(itemResult);
            }
            let items = resetData ? [] : this.state.items;
            if (limit) {
                items = items || Array(limit + offset);
                items
                    .slice(offset, offset + limit)
                    .filter((item) => item && item.id === undefined)
                    .forEach((item) => item &&
                    this._uidIndex[item.uid] &&
                    this._uidIndex[item.uid].id === undefined &&
                    delete this._uidIndex[item.uid]);
                items.splice(offset, limit, ...itemResult.slice(0, limit));
                this._setItems(items);
                if (data.length < limit) {
                    this.state.total = this.state.items?.length;
                }
            }
            else {
                if (items) {
                    items
                        .filter((item) => item && item.id === undefined)
                        .forEach((item) => item &&
                        this._uidIndex[item.uid] &&
                        this._uidIndex[item.uid].id === undefined &&
                        delete this._uidIndex[item.uid]);
                }
                this._setItems(itemResult);
                this.state.total = this.state.items?.length;
            }
            this.state.hasMore = hasMore;
            if (!hasMore) {
                this.state.total = this.state.items?.length;
            }
            if (total !== undefined) {
                this.state.total = total;
            }
        }
        same = same && query.limit === currentQuery.limit && query.offset === currentQuery.offset;
        if (same && resetData && hasMore) {
            this.state.status = LoadingStatus.PartialLoaded;
        }
        else {
            const values = Object.values(this._uidIndex);
            // need to calculate the status as we cannot be sure that there is not another running request
            if (values.find((item) => item.loadingStatus === LoadingStatus.Loading)) {
                this.state.status = LoadingStatus.PartialLoading;
            }
            else if (values.find((item) => item.loadingStatus === LoadingStatus.Fetching)) {
                this.state.status = LoadingStatus.PartialFetching;
            }
            else if (hasMore || Object.values(this.state.items || []).length !== (this.state.items || []).length) {
                this.state.status = LoadingStatus.PartialLoaded;
            }
            else {
                const undef = Object.values(this._uidIndex).find((item) => item === undefined);
                this.state.status = undef ? LoadingStatus.PartialLoaded : LoadingStatus.Loaded;
            }
        }
    }
    _buildItem(data, adaptee, _context) {
        const result = adaptee;
        const getId = (data) => {
            if (isNull(data)) {
                return _context?.position;
            }
            if (Array.isArray(data) && (typeof data[0] === 'string' || !isNaN(data[0]))) {
                return data[0];
            }
            else if (typeof data === 'string' || !isNaN(data)) {
                return data;
            }
            else if (!isNull(data.id)) {
                return data.id;
            }
            else {
                return _context?.position;
            }
        };
        const getText = (data) => {
            if (isNull(data)) {
                return undefined;
            }
            if (Array.isArray(data)) {
                return getText(data[1]);
            }
            else if (typeof data === 'string' || !isNaN(data) || data === true || data === false) {
                return String(data);
            }
            else if (!isNull(data.text)) {
                return String(data.text);
            }
            else {
                return undefined;
            }
        };
        ensureFieldValue(result, 'id', getId(data));
        ensureFieldValue(result, 'text', getText(data));
        const currentItem = this._idIndex[String(result.id)];
        const d = Array.isArray(data) ? data[0] : data;
        if (currentItem !== undefined) {
            if (currentItem.id === data) {
                // do not erase with new text or id, just get the current item
                return Object.assign({}, currentItem);
            }
            return Object.assign({}, currentItem, { data: d }, result);
        }
        else {
            const uid = generateUniqueId();
            return Object.assign({
                data: d,
                uid,
                loadingStatus: data !== undefined ? LoadingStatus.Loaded : LoadingStatus.NotInitialized,
            }, result);
        }
    }
    _execute(items, query, comparator, comparators, searcher) {
        try {
            let result = defaultQueryEngine(items, query, comparator, comparators, searcher);
            if (this.state.onQuerySuccess) {
                result = this.state.onQuerySuccess(result);
            }
            return result;
        }
        catch (e) {
            if (this.state.onQueryError) {
                this.state.onQueryError(DefaultBasicError.of(e));
            }
            throw e;
        }
    }
    _getId(data) {
        return this.adapt(data).id;
    }
    _indexDb(item, _context) {
        if (this._positionIndex[item.uid] !== undefined && this._db) {
            set(this._db, this._positionIndex[item.uid], item);
        }
    }
    _indexId(item, _context) {
        if (item.id !== undefined) {
            this._idIndex[item.id] = item;
        }
    }
    _indexItem(item, context) {
        this._indexUid(item, context);
        this._indexId(item, context);
        this._indexPosition(item, context);
        this._indexDb(item, context);
    }
    _indexPosition(item, context) {
        if (context?.position !== undefined) {
            this._positionIndex[item.uid] = `${context.position}`;
        }
    }
    _indexUid(item, _context) {
        this._uidIndex[item.uid] = item;
    }
    _isFiltered(query) {
        if (query.search !== undefined && query.search !== '') {
            return true;
        }
        const filter = query.filter;
        if (!filter) {
            return false;
        }
        return filter.criterias.length > 0;
    }
    _onLocationChange(location) {
        const nextQuery = this._parseLocation(location);
        if (!this._refreshing)
            return;
        this._refreshing = false;
        if (this.state.local) {
            this._setQuery(nextQuery);
            //if (location.relativeurl && this._cache[location.relativeurl]) {
            //  this._setItems(this._cache[location.relativeurl]);
            //} else {
            const queryEngine = this.state.queryEngine || this._execute.bind(this);
            this._setItems(queryEngine(this._db || [], nextQuery, this.state.comparator || defaultComparator, this.state.comparators || {}, this.state.searcher));
            if (location.relativeurl) {
                const keys = Object.keys(this._cache);
                if (keys.length > 100) {
                    delete this._cache[keys[0]];
                }
                this._cache[location.relativeurl] = this.state.items;
            }
            //}
        }
        else {
            const resetData = this.state.items ? this._resetData || shouldResetData(this.getQuery(), nextQuery) : false;
            if (resetData) {
                this._clearOffset(nextQuery);
            }
            this._setQuery(nextQuery);
            this[dispatch]({
                type: this[types]._fetch.actionType,
                payload: toPayload([nextQuery, resetData]),
            });
        }
    }
    _parseLocation(location) {
        return parseQuery(location.query || {});
    }
    _removeByMeta(field, by, target) {
        const items = this.setMeta(by, target, field, false);
        const removeUids = items.map((item) => item.uid);
        this.state[field] = (this.state[field] || []).filter((s) => !removeUids.includes(s));
        return items;
    }
    _removeFilter(query, filterId) {
        const filter = clone(formatFilter(query.filter));
        if (filter) {
            visitFilter(filter, (filter) => {
                for (const i in filter.criterias) {
                    if (filter.criterias[i].id === filterId) {
                        filter.criterias.splice(parseInt(i), 1);
                        return true;
                    }
                }
                return false;
            });
            query.filter = filter;
        }
    }
    _removeSortBy(query, sort) {
        const sortBy = formatSortBy(get(query, 'sortBy'), get(this.state, 'sortBy'));
        if (sortBy) {
            query.sortBy = sortBy.filter((s) => !isSameSortBy(sort, s));
        }
    }
    _removeSortById(query, id) {
        const sortBy = formatSortBy(get(query, 'sortBy'), get(this.state, 'sortBy'));
        if (sortBy) {
            query.sortBy = sortBy.filter((sort) => sort.id !== id);
        }
    }
    _setByMeta(field, by, target) {
        const current = this.state[field] || [];
        this.setMeta('uid', current, field, false);
        const items = this.setMeta(by, target, field, true);
        this.state[field] = items.map((item) => item.uid);
        return items;
    }
    _setFields(query, fields) {
        query.fields = fields;
    }
    _setFilter(query, filter) {
        query.filter = formatFilter(filter);
    }
    _setItems(items) {
        this.state.items = items;
    }
    _setLimit(query, limit) {
        query.limit = limit;
    }
    _setLimitOffset(query, limit, offset) {
        query.limit = limit;
        query.offset = offset;
    }
    _setLoading(options = {}) {
        if (isCollectionReady(this)) {
            if (this.state.local) {
                this.state.limit = options.limit;
                this.state.offset = options.offset;
            }
            else {
                const resetData = options.resetData ?? true;
                const resetLimit = options.resetLimit ?? true;
                if (options.status === undefined) {
                    options.status =
                        this.state.fetchOptions?.delayLoading !== undefined && this.state.fetchOptions?.delayLoading > 0
                            ? LoadingStatus.Fetching
                            : LoadingStatus.Loading;
                    this._resetData = resetData;
                }
                const setItemStatus = (item, status) => {
                    if (item === undefined) {
                        item = this.adapt(undefined);
                        item.loadingStatus = status;
                        this._indexItem(item);
                        return item;
                    }
                    item.loadingStatus = status;
                    this.setMeta('item', item, 'loadingStatus', status);
                    return Object.assign({}, item);
                };
                if (resetLimit) {
                    this.state.limit = options.limit;
                    this.state.offset = options.offset;
                }
                const offset = options.offset || 0;
                if (this.state.items) {
                    if (options.limit && !resetData) {
                        for (let i = offset; i < options.limit + offset; i++) {
                            this.state.items[i] = setItemStatus(this.state.items[i], options.status);
                        }
                        this.state.status = `${resetData ? '' : 'partial_'}${options.status}`;
                    }
                    else {
                        for (const i in this.state.items) {
                            this.state.items[i] = setItemStatus(this.state.items[i], options.status);
                        }
                        this.state.status = options.status;
                    }
                }
                else {
                    if (options.limit) {
                        this.state.items = Array(offset + options.limit);
                        for (let i = offset; i < offset + options.limit; i++) {
                            this.state.items[i] = setItemStatus(this.state.items[i], options.status);
                        }
                        this.state.status = `${resetData ? '' : 'partial_'}${options.status}`;
                    }
                    else {
                        // no items yet, so we are sure that the collection is either full loading of full fetching
                        this.state.status = options.status;
                    }
                }
            }
        }
    }
    _setQuery(query, force = true) {
        const nextFilter = formatFilter(query.filter);
        if (force || nextFilter) {
            this.state.filter = nextFilter;
        }
        const nextSortBy = formatSortBy(query.sortBy, get(this.state, 'sortBy'));
        if (force || query.sortBy !== undefined) {
            this.state.sortBy = nextSortBy;
        }
        if (force || query.sort)
            this.state.sort = query.sort;
        if (force || query.search)
            this.state.search = query.search;
        if (force || query.fields)
            this.state.fields = query.fields;
        if (force || query.offset)
            this.state.offset = query.offset;
        if (force || query.limit)
            this.state.limit = query.limit;
        if (force || query.params)
            this.state.params = query.params;
    }
    _setOffset(query, offset) {
        query.offset = offset;
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    _setParam(query, key, value) {
        query.params = Object.assign({}, query.params, { [key]: value });
    }
    _setParams(query, params) {
        query.params = params;
    }
    _setSearch(query, search) {
        query.search = search;
    }
    _setSort(query, dir) {
        query.sort = dir;
    }
    _setSortBy(query, sortBy) {
        query.sortBy = formatSortBy(sortBy, get(this.state, 'sortBy'));
    }
}
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_a = typeof B !== "undefined" && B) === "function" ? _a : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "addActive", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_b = typeof B !== "undefined" && B) === "function" ? _b : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "addDisabled", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "addFilter", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, Object, Object, Object, Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "addFilterCriteria", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_c = typeof B !== "undefined" && B) === "function" ? _c : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "addHighlighted", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_d = typeof B !== "undefined" && B) === "function" ? _d : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "addSelected", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Boolean]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "addSortBy", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "clearFields", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "clearFilter", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "clearLimit", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "clearOffset", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "clearParam", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "clearParams", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "clearSearch", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "clearSort", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "clearSortBy", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "filter", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, Number, Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "load", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "onSubscribe", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "query", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "reload", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Boolean, Boolean, Boolean]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "refresh", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_e = typeof B !== "undefined" && B) === "function" ? _e : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "removeActive", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_f = typeof B !== "undefined" && B) === "function" ? _f : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "removeDisabled", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "removeFilter", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_g = typeof B !== "undefined" && B) === "function" ? _g : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "removeHighlighted", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_h = typeof B !== "undefined" && B) === "function" ? _h : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "removeSelected", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "removeSortBy", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "reset", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "search", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_j = typeof B !== "undefined" && B) === "function" ? _j : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "setActive", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array, Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "setData", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_k = typeof B !== "undefined" && B) === "function" ? _k : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "setDisabled", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "setFields", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_l = typeof B !== "undefined" && B) === "function" ? _l : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "setHighlighted", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_m = typeof B !== "undefined" && B) === "function" ? _m : Object, Object, typeof (_o = typeof K !== "undefined" && K) === "function" ? _o : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "setMeta", null);
__decorate([
    reducer
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    ,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "setParam", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "setParams", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_p = typeof B !== "undefined" && B) === "function" ? _p : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "setSelected", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "setStatus", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "setUrl", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "sort", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "sortBy", null);
__decorate([
    saga(SagaEffect.Leading),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "startAutoRefresh", null);
__decorate([
    saga(SagaEffect.Leading),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "startFollow", null);
__decorate([
    saga(SagaEffect.Leading),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "stopAutoRefresh", null);
__decorate([
    saga(SagaEffect.Leading),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "stopFollow", null);
__decorate([
    saga(SagaEffect.Leading),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "_autoRefresh", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "_clearLimit", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "_clearSortBy", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "_clearSort", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "_disableDelayLoading", null);
__decorate([
    saga(SagaEffect.Latest)
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    ,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "_fetchOnce", null);
__decorate([
    saga(SagaEffect.Throttle, 'state.throttle', 1)
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    ,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Boolean]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "_fetch", null);
__decorate([
    saga(SagaEffect.Leading),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "_follow", null);
__decorate([
    reducer
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    ,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "_fetchError", null);
__decorate([
    reducer
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    ,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Boolean, Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "_fetchSuccess", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "_onLocationChange", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, typeof (_q = typeof B !== "undefined" && B) === "function" ? _q : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "_removeByMeta", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, typeof (_r = typeof B !== "undefined" && B) === "function" ? _r : Object, Object]),
    __metadata("design:returntype", Array)
], CollectionService.prototype, "_setByMeta", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "_setLoading", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", void 0)
], CollectionService.prototype, "_setQuery", null);
//# sourceMappingURL=CollectionService.js.map