import useLazyRef from '../core/useLazyRef';
import useTryRouter from '../core/useTryRouter';
import useTryStore from '../core/useTryStore';
import LocalRouter from '../router/LocalRouter';
import { get } from '../utils/object';
import { isCollection } from './utils';
const useCollectionInitialState = (dataSource, options = {}) => {
    const auth = get(useTryStore()?.getState(), 'auth.default');
    let router = useTryRouter();
    let dataOrUrl;
    if (isCollection(dataSource)) {
        // we are going to create a fake collection (because hooks cannot be conditionals)
        dataOrUrl = [];
        options = {};
    }
    else {
        dataOrUrl = dataSource;
    }
    const stateRef = useLazyRef(() => {
        if (!router || !options.mutateUrl) {
            router = new LocalRouter();
        }
        if (options.auth === true) {
            options.auth = auth;
        }
        else if (options.auth === false) {
            options.auth = undefined;
        }
        const fetchOptions = Object.assign({ delayLoading: 0 }, options);
        return {
            adapter: options.adapter,
            autoload: options.autoload,
            comparator: options.comparator,
            comparators: options.comparators,
            dataKey: options.dataKey || 'data',
            dataSource: dataOrUrl,
            fetchOptions,
            fields: options.initialFields,
            filter: options.initialFilter,
            fetchOnce: options.fetchOnce,
            hasDataSource: dataSource !== undefined,
            hasMoreKey: options.hasMoreKey || 'has_more',
            limit: options.initialLimit,
            local: Array.isArray(dataOrUrl) || options.fetchOnce,
            method: options.method,
            mutateUrl: options.mutateUrl,
            onQueryError: options.onQueryError,
            onQuerySuccess: options.onQuerySuccess,
            offset: options.initialOffset,
            params: options.initialParams,
            queryEngine: options.queryEngine,
            router,
            search: options.initialSearch,
            searcher: options.searcher,
            serializer: options.serializer,
            sort: options.initialSort,
            sortBy: options.initialSortBy,
            throttle: options.throttle,
            totalKey: options.totalKey || 'total',
            url: Array.isArray(dataOrUrl) ? undefined : dataOrUrl,
        };
    });
    return stateRef.current;
};
export default useCollectionInitialState;
//# sourceMappingURL=useCollectionInitialState.js.map