import { useEffect, useMemo, useRef } from 'react';
import { LoadingStatus } from './typings';
import useCollectionService from './useCollectionService';
import { isCollection } from './utils';
export const collectionProxyHandler = {
    get: function (target, prop, receiver) {
        if (prop === 'asService') {
            return () => target;
        }
        else {
            return Reflect.get(target, prop, receiver);
        }
    },
};
const useCollectionProxy = (dataSource, ctor, initialState) => {
    const [, service] = useCollectionService(ctor, initialState);
    const dsRef = useRef(dataSource);
    const ds = isCollection(dataSource) ? dataSource : dsRef.current;
    const collectionProxy = useMemo(() => {
        if (isCollection(ds)) {
            return ds;
        }
        const proxy = new Proxy(service, collectionProxyHandler);
        return proxy;
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [ds, service.state]);
    useEffect(() => {
        if (initialState.dataSource === undefined &&
            Array.isArray(dataSource) &&
            collectionProxy.status === LoadingStatus.NotInitialized) {
            collectionProxy.setData(dataSource);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    });
    return collectionProxy;
};
export default useCollectionProxy;
//# sourceMappingURL=useCollectionProxy.js.map