import { debounce as debounceSaga, take, takeEvery, takeLatest, takeLeading, throttle as throttleSaga, } from 'redux-saga/effects';
import 'reflect-metadata';
import { SagaEffect } from '../types/saga';
import { serviceClass } from '../types/service';
import { ID } from '../types/symbol';
const uids = new Set();
let counter = 0;
const uniqueLabel = (label) => {
    if (!label) {
        label = `Label$${++counter}`;
    }
    else if (uids.has(label)) {
        label = `${label}$${++counter}`;
    }
    else {
        uids.add(label);
    }
    return label;
};
export const asReducer = function (func) {
    func.reducer = true;
    return func;
};
export const inject = (ctor) => {
    const result = {
        [serviceClass]: ctor,
    };
    return result;
};
// eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
export function reducer(target, propertyKey) {
    target[propertyKey].reducer = true;
}
// eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
export const service = function (target) {
    if (!target[ID]) {
        target[ID] = uniqueLabel(target.name);
    }
};
export const serial = function (func) {
    func.saga = {};
    return func;
};
export const latest = function (func) {
    func.saga = {
        effect: takeLatest,
    };
    return func;
};
export const every = function (func) {
    func.saga = {
        effect: takeEvery,
    };
    return func;
};
export const leading = function (func) {
    func.saga = {
        effect: takeLeading,
    };
    return func;
};
export const debounce = function (func, delay, defaultDelay = 200) {
    func.saga = {
        effect: debounceSaga,
        delay,
        defaultDelay,
    };
    return func;
};
export const throttle = function (func, delay, defaultDelay = 200) {
    func.saga = {
        effect: throttleSaga,
        func,
        delay,
        defaultDelay,
    };
    return func;
};
const effects = {
    [SagaEffect.Debounce]: debounceSaga,
    [SagaEffect.Throttle]: throttleSaga,
    [SagaEffect.Every]: takeEvery,
    [SagaEffect.Leading]: takeLeading,
    [SagaEffect.Latest]: takeLatest,
    [SagaEffect.Serial]: take,
};
export function saga(type, delay, defaultDelay = 0) {
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    return function (target, propertyKey) {
        target[propertyKey].saga = {
            effect: type ? effects[type] : take,
            delay,
            defaultDelay,
        };
    };
}
//# sourceMappingURL=annotations.js.map