import { useEffect } from 'react';
const useEventListener = (eventName, handler, capture = false) => {
    // Create a ref that stores handler
    useEffect(() => {
        // Create event listener that calls handler function stored in ref
        const eventListener = (event) => handler(event);
        // Add event listener
        document.addEventListener(eventName, eventListener, capture);
        // Remove event listener on cleanup
        return () => {
            document.removeEventListener(eventName, eventListener);
        };
    }, [handler, eventName, capture]);
};
class EventLock {
    locks = {};
    lock(type, id) {
        const locks = this.locks[type] || [];
        if (!locks.find((l) => l === id)) {
            locks.unshift(id);
        }
        this.locks[type] = locks;
    }
    unlock(type, id) {
        const locks = this.locks[type] || [];
        const index = locks.findIndex((l) => l === id);
        if (index >= 0) {
            locks.splice(index, 1);
        }
        this.locks[type] = locks;
    }
    isLockedBy(type, id) {
        if (this.locks[type] === undefined)
            return true; // the lock if free for everybody
        return this.locks[type][0] === id;
    }
}
export const eventLocks = new EventLock();
export default useEventListener;
//# sourceMappingURL=useEventListener.js.map