import { useMemo } from 'react';
import { get } from '../utils/object';
import { isFunction } from '../utils/type';
import useLazyRef from './useLazyRef';
const getObjectProps = (object) => {
    const props = [];
    let obj = object;
    do {
        props.push(...Object.getOwnPropertyNames(obj));
    } while ((obj = Object.getPrototypeOf(obj)));
    return props;
};
const useObjectProxy = (object, options) => {
    const mutableKeys = options.mutables || [];
    const pickKeys = options.pick || [];
    const omitKeys = options.omit || [];
    const immutables = useLazyRef(() => {
        let keys = pickKeys.length > 0 ? pickKeys : getObjectProps(object);
        keys = keys.filter((k) => !(mutableKeys.includes(k) || omitKeys.includes(k)));
        return keys.reduce((accumulator, prop) => {
            if (isFunction(get(object, prop))) {
                accumulator[prop] = get(object, prop).bind(object);
            }
            else {
                accumulator[prop] = get(object, prop);
            }
            return accumulator;
        }, {});
    });
    const mutables = mutableKeys.map((k) => get(object, k));
    const proxy = useMemo(() => {
        const mutables = mutableKeys.reduce((accumulator, prop) => {
            if (isFunction(get(object, prop))) {
                accumulator[prop] = get(object, prop).bind(object);
            }
            else {
                accumulator[prop] = get(object, prop);
            }
            return accumulator;
        }, {});
        return Object.assign({}, immutables.current, mutables);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, mutables);
    return proxy;
};
export default useObjectProxy;
//# sourceMappingURL=useObjectProxy.js.map