import useAppContext from '../app/useAppContext';
import useRouter from '../app/useRouter';
import { asResultCallback } from '../app/utils';
import useLazyRef from '../core/useLazyRef';
import { clone } from '../utils/object';
export function useFetchOptions(options) {
    const appContext = useAppContext();
    const router = useRouter();
    const fetchOptionsRef = useLazyRef(() => {
        const fetchOptions = clone(options);
        if (options.onError) {
            fetchOptions.onFetchError = (e) => {
                const callbackError = asResultCallback(options.onError, router, appContext);
                if (callbackError) {
                    callbackError(e);
                }
            };
        }
        if (options.onSuccess) {
            fetchOptions.onFetchSuccess = (result) => {
                const callbackSuccess = asResultCallback(options.onSuccess, router, appContext);
                if (callbackSuccess) {
                    callbackSuccess(result);
                }
            };
        }
        return fetchOptions;
    });
    return fetchOptionsRef.current;
}
//# sourceMappingURL=useFetchOptions.js.map