import { useCallback, useEffect, useRef } from 'react';
import FetchService from '../core/FetchService';
import useService from '../core/useService';
import { useFetchOptions } from './useFetchOptions';
const useGet = (url, options = {}) => {
    const pollingTaskRef = useRef(null);
    const optionsRef = useRef(options);
    const fetchOptions = useFetchOptions(options);
    const [state, service] = useService(FetchService, {
        loading: false,
        result: optionsRef.current.defaultValue,
    });
    const refresh = useCallback(() => {
        if (url) {
            if (pollingTaskRef !== null) {
                pollingTaskRef.current?.cancel();
                pollingTaskRef.current = null;
            }
            if (optionsRef.current.pollingMs) {
                service.poll(url, optionsRef.current.pollingMs, fetchOptions).then((task) => {
                    pollingTaskRef.current = task;
                });
            }
            else {
                service.get(url, fetchOptions);
            }
        }
    }, [url, service, fetchOptions]);
    useEffect(() => {
        refresh();
        return () => {
            if (pollingTaskRef !== null) {
                pollingTaskRef.current?.cancel();
                pollingTaskRef.current = null;
            }
        };
    }, [refresh]);
    return [state.result, state.loading || false, refresh];
};
export default useGet;
//# sourceMappingURL=useGet.js.map