import { __decorate, __metadata } from "tslib";
import { fork } from 'redux-saga/effects';
import { reducer, saga, service } from '../core/annotations';
import DefaultService from '../core/Service';
import { SagaEffect } from '../types/saga';
let AsyncBindService = class AsyncBindService extends DefaultService {
    setLoading(isLoading) {
        this.state.loading = isLoading;
    }
    success(result) {
        this.state.result = result;
        this.state.loading = false;
        this.state.error = undefined;
    }
    error(error) {
        this.state.loading = false;
        this.state.error = error;
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    *execute(asyncMethod, dependencies) {
        try {
            const task = yield this.forkAsyncBind(asyncMethod, dependencies);
            const error = task.error();
            if (error) {
                yield this.error(error);
            }
            else {
                yield this.success(task.result());
            }
        }
        catch (e) {
            yield this.error(e);
        }
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    *forkAsyncBind(asyncMethod, dependencies) {
        const task = yield fork(asyncMethod, ...dependencies);
        const async = task.isRunning();
        if (async) {
            yield this.setLoading(true);
        }
        return task;
    }
};
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Boolean]),
    __metadata("design:returntype", void 0)
], AsyncBindService.prototype, "setLoading", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], AsyncBindService.prototype, "success", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Error]),
    __metadata("design:returntype", void 0)
], AsyncBindService.prototype, "error", null);
__decorate([
    saga(SagaEffect.Latest),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Function, Array]),
    __metadata("design:returntype", void 0)
], AsyncBindService.prototype, "execute", null);
AsyncBindService = __decorate([
    service
], AsyncBindService);
export default AsyncBindService;
//# sourceMappingURL=AsyncBindService.js.map