import { Task } from '@redux-saga/types';
import DefaultService from '../core/Service';
import { ValidationStatus } from '../types/form';
import { AnonymousKeyObject, AnonymousObject, AnonymousPathObject, ArrayElement, NestedKeyOf, PathType } from '../types/object';
import ContainerValidation from './ContainerValidation';
import FieldValidation from './FieldValidation';
import { Field, FieldOptions, FieldProps, FormConfig, FormDecorator, FormDecoratorOptions, FormDisplayerField, FormListenerProps, FormListenerType, FormMetadata, FormMetadataListener, FormState, FormSubmitListener, FormValidationListener, FormValueListener, PlaceholderField, ValidationCode, ValidationResult, Validator, ValidatorAsyncFunction, ValidatorSyncFunction } from './typings';
export declare const FORM_GLOBAL_VALIDATION_KEY = "__$FORM$GLOBAL_VALIDATION_KEY__";
export default class FormService<T extends object = any> extends DefaultService<FormState<T>> {
    prevValues: AnonymousObject;
    prevValidations: AnonymousObject<FieldValidation>;
    prevSubmitting: boolean;
    prevMetadata: AnonymousObject<FormMetadata>;
    triggered: AnonymousObject<boolean>;
    fields: AnonymousKeyObject<T, Field<T>>;
    decorators: AnonymousObject<FormDecorator>;
    listeners: {
        [k in FormListenerType]: AnonymousObject<FormListenerProps[]>;
    };
    pendingDispatch: {
        [fieldName: string]: Set<string>;
    };
    protected fieldIndex: AnonymousObject;
    protected watchIndex: AnonymousObject;
    protected validations: AnonymousObject<string>[];
    protected listenerIndex: {
        [id: string]: AnonymousObject<FormListenerProps>;
    };
    protected placeholderFields: AnonymousObject<PlaceholderField<T>>;
    config: FormConfig;
    initializing: boolean;
    protected defaultValues: AnonymousPathObject<T>;
    protected defaultMetadata: AnonymousObject<FormMetadata>;
    constructor();
    add<K extends NestedKeyOf<T>>(fieldArrayName: K, initialValue?: Partial<ArrayElement<PathType<T, K>>>): void;
    addField(field: Field<T>): void;
    addValidator(fieldName: NestedKeyOf<T>, validatorName: string, validator: Validator): void;
    getDisplayerTree(): AnonymousObject<FormDisplayerField>;
    clearError(fieldName: NestedKeyOf<T>, validatorName?: string, compile?: boolean): void;
    clearValidation(fieldName: NestedKeyOf<T> | undefined, validatorName: string | undefined, code: ValidationCode, compile?: boolean): void;
    clearWarning(fieldName: NestedKeyOf<T>, validatorName?: string, compile?: boolean): void;
    compileValidations(fieldNames: undefined | NestedKeyOf<T>[] | NestedKeyOf<T>, force?: boolean): void;
    decorator(name: string, options?: FormDecoratorOptions): FormDecorator;
    delayLoading(delay_ms?: number): Generator<void | import("redux-saga/effects").CallEffect<true>, void, unknown>;
    disable(fieldOrDecoratorName: string, match?: boolean): void;
    disableValidator(fieldName: NestedKeyOf<T>, validatorName: string): void;
    enable(fieldOrDecoratorName: string, match?: boolean): void;
    enableValidator(fieldName: NestedKeyOf<T>, validatorName: string): void;
    /**
     *  this method is an helper to quickly register a field from a component. It will return
     *   - name
     *   - value
     *   - onChange
     *   - onFocus
     *   - onBlur
     *
     * @param {string} name - the name of the field. Must be related to a key in values
     * @param {Object} validators - a object of validators
     *
     * @return {Object} a list of props for the field
     *                    - name
     *                    - value
     *                    - onChange
     *                    - onFocus
     *                    - onBlur
     */
    field(name: NestedKeyOf<T>, validators?: AnonymousObject<Validator>, options?: FieldOptions): FieldProps;
    isField(fieldOrDecoratorName: string): fieldOrDecoratorName is NestedKeyOf<T>;
    isReadOnlyField(fieldName: NestedKeyOf<T>): boolean | undefined;
    getContainerFieldValidation(validations: AnonymousObject<FieldValidation>, fields: AnonymousKeyObject<T, Field>, prefix?: string, touchedOnly?: boolean, includeGlobal?: boolean): ContainerValidation;
    getContext(key?: string, defaultValue?: any): any;
    protected _getSubFieldNames(fieldName: NestedKeyOf<T>): NestedKeyOf<T>[];
    getMetadata<K extends keyof FormMetadata>(fieldOrDecoratorName: string, key: K): FormMetadata[K] | undefined;
    getMetadatas(fieldOrDecoratorName: string): FormMetadata | undefined;
    obfuscate(value: any, fieldName?: string): any;
    protected _getSubWatchs(watch: NestedKeyOf<T>): (NestedKeyOf<T> | '')[];
    getValidation: (fieldName?: NestedKeyOf<T>, touchedOnly?: boolean) => FieldValidation | ContainerValidation;
    protected _getValidation(fieldName: NestedKeyOf<T> | undefined): FieldValidation;
    getValue(fieldName?: undefined | '', defaultValue?: T): T;
    getValue<K extends NestedKeyOf<T>>(fieldName: K, defaultValue: Exclude<PathType<T, K>, undefined | null>): Exclude<PathType<T, K>, undefined | null>;
    getValue<K extends NestedKeyOf<T>>(fieldName: K, defaultValue?: PathType<T, K>): PathType<T, K> | undefined;
    hasValidation(fieldName: NestedKeyOf<T> | undefined, validatorName: string, code: ValidationCode, message?: string): boolean;
    hide(fieldOrDecoratorName: string, match?: boolean): void;
    /**
     *  Register a decorator. Unlike a field, a decorator has no value but can have metadata (like disabled / visible)
     *
     * @param {string} name - the name of the decorator. Must not be in conflict with a field name
     *
     * @return {void}
     */
    initDecorator(name: string, options?: FormDecoratorOptions): FormDecorator;
    /**
     *  Register a field and return three listeners
     *   - onChange
     *   - onFocus
     *   - onBlur
     *
     * @param {string} name - the name of the field. Must be related to a key in values
     * @param {Validator[]} validators - a object of validators
     * @param {FieldOptions} options
     *
     * @return {Object} a list of listeners for the field and its name
     *                    - name
     *                    - onChange
     *                    - onFocus
     *                    - onBlur
     */
    initField(name: NestedKeyOf<T>, validators?: AnonymousObject<Validator>, options?: FieldOptions): FieldProps;
    insert(fieldArrayName: NestedKeyOf<T>, index: number, initialValue?: {}): void;
    isTouched(fieldName: NestedKeyOf<T>): boolean;
    loadInitialValues(fetcher: string | (() => T | Promise<T>)): Generator<T | import("redux-saga/effects").ForkEffect<Generator<void | import("redux-saga/effects").CallEffect<true>, void, unknown>> | Promise<any> | import("redux-saga/effects").CancelEffect, void, (Task<any> | null) & T>;
    offChange(type: FormListenerType, id: string): void;
    onChange(type: FormListenerType, id: string, listener: FormValueListener | FormValidationListener | FormSubmitListener, watchs: string[] | string, once: boolean): void;
    onMount(): void;
    offMetadataChange(id: string): void;
    onMetadataChange(id: string, listener: FormMetadataListener, watchs: string[] | string, once?: boolean): void;
    offSubmittingChange(id: string): void;
    onSubmittingChange(id: string, listener: FormSubmitListener, once?: boolean): void;
    offValidationChange(id: string): void;
    onValidationChange(id: string, listener: FormValidationListener, watchs: string[] | string, once?: boolean): void;
    offValueChange(id: string): void;
    onValueChange(id: string, listener: FormValueListener, watchs: string[] | string, once?: boolean): void;
    remove(fieldArrayName: NestedKeyOf<T>, index: number, changeValue?: boolean): void;
    removeValidator(fieldName: NestedKeyOf<T>, validatorName: string): void;
    reset(): void;
    serializeValidationCode(code: ValidationCode): ValidationStatus;
    setContext(key: string | undefined, value: any): any;
    setError(fieldName: NestedKeyOf<T> | undefined, validatorName: string, message?: string, match?: boolean): boolean;
    setInitialValues(values: Partial<T>): void;
    setLoading(loading: boolean, fetching: boolean): void;
    setMetadata<K extends keyof FormMetadata>(fieldOrDecoratorName: string, key: K, value: FormMetadata[K], force?: boolean): void;
    setMetadatas(metadatas: AnonymousObject<FormMetadata>): void;
    setOK(fieldName: NestedKeyOf<T> | undefined, validatorName: string): boolean;
    setOrClearValidation(code: ValidationCode, fieldName: NestedKeyOf<T> | undefined, validatorName: string, message?: string, match?: boolean): boolean;
    setPendingValidation(fieldName: NestedKeyOf<T> | undefined, validatorName: string, pending?: boolean): boolean;
    setSubmitting(submitting: boolean): void;
    setValidation(fieldName: NestedKeyOf<T> | undefined, validatorName: string, code: ValidationCode, message?: string, compile?: boolean): void;
    setValue<K extends NestedKeyOf<T>>(fieldName: K, value: PathType<T, K>): void;
    setValues(values: AnonymousPathObject<T>): void;
    setWarning(fieldName: NestedKeyOf<T> | undefined, validatorName: string, message?: string, match?: boolean): boolean;
    show(fieldOrDecoratorName: string, match?: boolean): void;
    submit(): Generator<void | ContainerValidation, void, {
        code: any;
        fields: any;
    }>;
    touch(fieldName: NestedKeyOf<T>): void;
    touchAll(): void;
    validateSync(fieldName: NestedKeyOf<T>, validatorName: string, validator: ValidatorSyncFunction, value: any): void;
    validateAsync(fieldName: NestedKeyOf<T>, validatorName: string, validator: ValidatorAsyncFunction, value: any): Generator<void | Promise<ValidationResult>, void, ValidationResult>;
    validateAll(values: AnonymousPathObject<T>): AnonymousKeyObject<T, FieldValidation>;
}
