var _b, _c, _d;
import { __decorate, __metadata } from "tslib";
import { cancel, delay, fork } from 'redux-saga/effects';
import DefaultService from '../core/Service';
import { reducer, saga, service } from '../core/annotations';
import { asyncGet } from '../core/xhr';
import { ValidationStatus } from '../types/form';
import { SagaEffect } from '../types/saga';
import { clone, del, get, isObject, set, simpleMergeDeep } from '../utils/object';
import { generateUniqueId } from '../utils/string';
import ContainerValidation from './ContainerValidation';
import FieldValidation, { defaultValidation } from './FieldValidation';
import { TouchOn, ValidationCode, } from './typings';
import { getNonIndexedProp } from './utils';
export const FORM_GLOBAL_VALIDATION_KEY = '__$FORM$GLOBAL_VALIDATION_KEY__';
let FormService = class FormService extends DefaultService {
    // keep the previous values (once the changes have been triggered, this field contains the current values)
    prevValues = {};
    // keep the previous validations (once the changes have been triggered, this field contains the current validations)
    prevValidations = {};
    // keep the previous submitting value (once the changes have been triggered, this field contains the current submitting value)
    prevSubmitting = false;
    // keep the previous metadata (once the changes have been triggered, this field contains the current metadata)
    prevMetadata = {};
    // keep all fields that has been triggered at least once (used to be sure that all field initialization are listened by listeners)
    triggered = {};
    fields;
    decorators;
    listeners;
    pendingDispatch;
    fieldIndex;
    watchIndex;
    // global validations, not related to a field
    // can be used to set the form in error related to an external event (not enough permissions, ...)
    validations = [];
    listenerIndex;
    // use when a rule change a field that was not yet initialized
    placeholderFields;
    config = {
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        onSubmit: () => { },
    };
    initializing = true;
    defaultValues = {};
    defaultMetadata = {};
    constructor() {
        super();
        this.fields = {};
        this.placeholderFields = {};
        this.decorators = {};
        this.listeners = {
            valueChange: {},
            validationChange: {},
            submittingChange: {},
            metadataChange: {},
        };
        this.pendingDispatch = {
            valueChange: new Set(),
            validationChange: new Set(),
            submittingChange: new Set(),
            metadataChange: new Set(),
        };
        this.fieldIndex = {};
        this.watchIndex = {};
        this.listenerIndex = {};
    }
    add(fieldArrayName, initialValue) {
        let arrayValue = get(this.state.values, fieldArrayName, []);
        if (arrayValue === undefined || arrayValue === null || !Array.isArray(arrayValue)) {
            arrayValue = [];
        }
        this.setValue(fieldArrayName, arrayValue.concat([initialValue]));
    }
    addField(field) {
        if (!this.fields[field.context.name]) {
            this.fields[field.context.name] = field;
            const currentIndex = get(this.fieldIndex, field.context.name);
            if (currentIndex === undefined) {
                set(this.fieldIndex, field.context.name, true);
            }
        }
    }
    addValidator(fieldName, validatorName, validator) {
        // get current value
        const field = this.fields[fieldName];
        if (field) {
            field.validators[validatorName] = validator;
        }
        else {
            set(this.placeholderFields, `${fieldName}.validators.${validatorName}`, validator);
        }
        const validations = this.validateAll({
            [fieldName]: this.getValue(fieldName),
        });
        this.compileValidations(Object.keys(validations));
    }
    getDisplayerTree() {
        const treeObject = {};
        for (const fieldName of Object.keys(this.fields || {})) {
            const field = this.fields[fieldName];
            let obj = treeObject;
            let hidden = false;
            for (const container of get(field, 'containers', [])) {
                const decorator = this.decorators[container] ?? get(this.fields, container);
                const metadata = this.state.metadata[container];
                if (!(metadata?.visible ?? true)) {
                    hidden = true;
                    break;
                }
                else {
                    if (obj[container] === undefined) {
                        obj[container] = {
                            name: container,
                            Displayer: decorator.Displayer,
                            children: {},
                        };
                    }
                    obj = obj[container].children;
                }
            }
            ;
            const metadata = this.state.metadata[fieldName];
            if (!hidden && (metadata?.visible ?? true)) {
                obj[fieldName] = Object.assign({
                    name: fieldName,
                    Displayer: field.Displayer,
                    children: {},
                });
            }
        }
        return treeObject;
    }
    clearError(fieldName, validatorName, compile = true) {
        this.clearValidation(fieldName, validatorName, ValidationCode.Error, compile);
    }
    clearValidation(fieldName, validatorName, code, compile = true) {
        if (fieldName === undefined) {
            del(this.validations, validatorName ? `${code}.${validatorName}` : code);
        }
        else {
            del(this.fields[fieldName], validatorName ? `validations.${code}.${validatorName}` : `validations.${code}`);
        }
        if (compile) {
            this.compileValidations(fieldName);
        }
    }
    clearWarning(fieldName, validatorName, compile = true) {
        this.clearValidation(fieldName, validatorName, ValidationCode.Warning, compile);
    }
    compileValidations(fieldNames, force = false) {
        if (fieldNames === undefined) {
            const previousValidation = this.state.validations[FORM_GLOBAL_VALIDATION_KEY];
            const nextValidation = this._getValidation(undefined);
            if (force || !nextValidation.equals(previousValidation)) {
                this.state.validations[FORM_GLOBAL_VALIDATION_KEY] = nextValidation;
                this.pendingDispatch.validationChange.add('');
            }
        }
        else {
            if (!Array.isArray(fieldNames)) {
                fieldNames = [fieldNames];
            }
            fieldNames.forEach((fieldName) => {
                if (this.fields[fieldName]) {
                    const previousValidation = this.state.validations[fieldName];
                    const nextValidation = this._getValidation(fieldName);
                    if (force || !nextValidation.equals(previousValidation)) {
                        this.state.validations[fieldName] = nextValidation;
                        this.pendingDispatch.validationChange.add(fieldName);
                        this.pendingDispatch.validationChange.add('');
                    }
                }
            });
        }
    }
    decorator(name, options = {}) {
        return this.initDecorator(name, options);
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    *delayLoading(delay_ms) {
        yield this.setLoading(delay_ms ? false : true, true);
        if (delay_ms) {
            yield delay(delay_ms);
            yield this.setLoading(true, true);
        }
    }
    disable(fieldOrDecoratorName, match = true) {
        // ignore if the field is not editable
        if (this.isField(fieldOrDecoratorName) && this.isReadOnlyField(fieldOrDecoratorName)) {
            return;
        }
        this.setMetadata(fieldOrDecoratorName, 'disabled', match);
    }
    disableValidator(fieldName, validatorName) {
        let field = this.fields[fieldName];
        if (!field) {
            if (!this.placeholderFields[fieldName]) {
                this.placeholderFields[fieldName] = {};
            }
            field = this.placeholderFields[fieldName];
            if (!field.validators) {
                field.validators = {};
            }
            if (!field.validators[validatorName]) {
                field.validators[validatorName] = {};
            }
        }
        if (field && field.validators) {
            let validator = field.validators[validatorName];
            if (validator && typeof validator !== 'object') {
                field.validators[validatorName] = {
                    validator: field.validators[validatorName],
                    disabled: false,
                };
            }
            validator = field.validators[validatorName];
            if (validator && !validator.disabled) {
                validator.disabled = true;
                this.clearValidation(fieldName, validatorName, ValidationCode.Error, true);
            }
        }
    }
    enable(fieldOrDecoratorName, match = true) {
        // ignore if the field is not editable
        if (this.isField(fieldOrDecoratorName) && this.isReadOnlyField(fieldOrDecoratorName)) {
            return;
        }
        this.setMetadata(fieldOrDecoratorName, 'disabled', !match);
    }
    enableValidator(fieldName, validatorName) {
        let field = this.fields[fieldName];
        if (!field) {
            if (!this.placeholderFields[fieldName]) {
                this.placeholderFields[fieldName] = {};
            }
            field = this.placeholderFields[fieldName];
            if (!field.validators) {
                field.validators = {};
            }
            if (!field.validators[validatorName]) {
                field.validators[validatorName] = {};
            }
        }
        if (field && field.validators) {
            let validator = field.validators[validatorName];
            if (validator && typeof validator !== 'object') {
                field.validators[validatorName] = {
                    validator: field.validators[validatorName],
                    disabled: false,
                };
            }
            validator = field.validators[validatorName];
            if (validator && validator.disabled && this.fields[fieldName]) {
                validator.disabled = false;
                this.validateSync(fieldName, validatorName, validator.validator, this.getValue(fieldName));
                this.compileValidations(fieldName);
            }
        }
    }
    /**
     *  this method is an helper to quickly register a field from a component. It will return
     *   - name
     *   - value
     *   - onChange
     *   - onFocus
     *   - onBlur
     *
     * @param {string} name - the name of the field. Must be related to a key in values
     * @param {Object} validators - a object of validators
     *
     * @return {Object} a list of props for the field
     *                    - name
     *                    - value
     *                    - onChange
     *                    - onFocus
     *                    - onBlur
     */
    field(name, validators = {}, options = {}) {
        const field = this.initField(name, validators, options);
        field.value = get(this.state.values, name, options.defaultValue === undefined ? '' : options.defaultValue);
        return field;
    }
    isField(fieldOrDecoratorName) {
        return Object.keys(this.fields).includes(fieldOrDecoratorName);
    }
    isReadOnlyField(fieldName) {
        const metadata = this.state.metadata[fieldName] || {};
        return metadata.readOnly || (this.config.reconfigure && !metadata.editable);
    }
    getContainerFieldValidation(validations, fields, prefix = '', touchedOnly = true, includeGlobal = false) {
        // compile the validations to get the status
        let messages = [];
        const result = new ContainerValidation('', ValidationStatus.None, ValidationCode.None, {});
        const keys = Object.keys(validations);
        for (const fieldName of keys.filter((k) => k.startsWith(prefix) && k !== FORM_GLOBAL_VALIDATION_KEY)) {
            const field = fields[fieldName];
            if (field !== undefined && (!touchedOnly || field.touched)) {
                const validation = validations[fieldName];
                if (validation.code <= result.code && validation.code < ValidationCode.None) {
                    if (validation.code < result.code) {
                        result.status = validation.status;
                        result.code = validation.code;
                        result.fields = {};
                        messages = [];
                    }
                    result.fields[fieldName] = validation.message;
                    if (validation.message) {
                        messages.push(`<${fieldName}>: ${validation.message}`);
                    }
                }
            }
        }
        if (includeGlobal) {
            const validation = validations[FORM_GLOBAL_VALIDATION_KEY];
            if (validation && validation.code <= result.code && validation.code < ValidationCode.None) {
                if (validation.code < result.code) {
                    result.status = validation.status;
                    result.code = validation.code;
                    result.fields = {};
                    messages = [];
                }
                result.fields[FORM_GLOBAL_VALIDATION_KEY] = validation.message;
                if (validation.message) {
                    messages.push(`${validation.message}`);
                }
            }
        }
        if (messages.length > 0) {
            result.message = messages.join('\n');
        }
        return result;
    }
    getContext(key, defaultValue) {
        if (key === undefined || key === '') {
            return this.state.context ?? defaultValue;
        }
        return get(this.state.context, key, defaultValue);
    }
    _getSubFieldNames(fieldName) {
        let result = [];
        const index = get(this.fieldIndex, fieldName);
        if (index) {
            if (Array.isArray(index)) {
                for (const i in index) {
                    result = result.concat(this._getSubFieldNames(`${fieldName}.${i}`));
                }
            }
            else if (isObject(index)) {
                Object.keys(index).forEach((childFieldName) => {
                    result = result.concat(this._getSubFieldNames(`${fieldName}.${childFieldName}`));
                });
            }
            if (this.fields[fieldName]) {
                result.push(fieldName);
            }
        }
        return result;
    }
    getMetadata(fieldOrDecoratorName, key) {
        return get(this.state.metadata, `${fieldOrDecoratorName}.${key}`);
    }
    getMetadatas(fieldOrDecoratorName) {
        return get(this.state.metadata, fieldOrDecoratorName);
    }
    obfuscate(value, fieldName) {
        value = clone(value); // do not touch the original
        const obfuscate = (v, ns) => {
            const field = this.fields[ns];
            if (field && field.protected) {
                set(value, ns, '*************');
            }
            else if (Array.isArray(v)) {
                v.forEach((item, index) => obfuscate(item, `${ns}.${index}`));
            }
            else if (typeof v === 'object') {
                Object.keys(v).forEach((k) => obfuscate(value[k], `${ns ? `${ns}.` : ''}${k}`));
            }
        };
        obfuscate(value, fieldName || '');
        return value;
    }
    _getSubWatchs(watch) {
        const nonIndexedWatch = getNonIndexedProp(watch);
        let result = [];
        // check if the index if something listens on the key "watch"
        const index = watch === '' ? this.watchIndex[''] : get(this.watchIndex, watch);
        if (index) {
            // direct listener on "addresses" are also alerted
            result.push(watch);
        }
        if (nonIndexedWatch !== undefined) {
            const nonIndex = get(this.watchIndex, nonIndexedWatch);
            if (nonIndex) {
                result.push(watch);
            }
        }
        // if the value "addresses" changes, addresses.0.street should be alerted, .....
        result = result.concat(this._getSubFieldNames(watch));
        // if addresses.0.street is changed, addresses should be alerted (becasue the object has been changed)
        // we will also alert adresses.street but it would be done in form/index.tsx
        while (watch.includes('.')) {
            watch = watch.split('.').slice(0, -1).join('.');
            const index = watch === '' ? this.watchIndex[''] : get(this.watchIndex, watch);
            if (index) {
                result.push(watch);
            }
        }
        if (this.watchIndex['']) {
            result.push('');
        }
        return result;
    }
    getValidation = (fieldName, touchedOnly = true) => {
        const getFieldValidation = (fieldName) => {
            const field = this.fields[fieldName];
            if (field !== undefined) {
                if (touchedOnly) {
                    return field.touched ? this.state.validations[fieldName] || defaultValidation : defaultValidation;
                }
                else {
                    return this.state.validations[fieldName] || defaultValidation;
                }
            }
            else {
                return this.getContainerFieldValidation(this.state.validations, this.fields, fieldName, touchedOnly);
            }
        };
        if (fieldName === undefined || fieldName === '') {
            return this.getContainerFieldValidation(this.state.validations, this.fields, '', touchedOnly, true);
        }
        else {
            return getFieldValidation(fieldName);
        }
    };
    _getValidation(fieldName) {
        let validations;
        if (fieldName === undefined) {
            validations = this.validations;
        }
        else if (this.fields[fieldName] !== undefined) {
            validations = this.fields[fieldName]?.validations;
        }
        if (validations !== undefined) {
            for (const code in validations) {
                const iCode = parseInt(code);
                for (const id in validations[iCode]) {
                    return new FieldValidation(validations[iCode][id], this.serializeValidationCode(iCode), iCode);
                }
            }
            return defaultValidation;
        }
        return defaultValidation;
    }
    getValue(fieldName, defaultValue) {
        return get(this.state.values, fieldName, defaultValue);
    }
    hasValidation(fieldName, validatorName, code, message) {
        if (fieldName === undefined) {
            const validation = get(this.validations, `${code}.${validatorName}`);
            if (!message) {
                return validation !== undefined;
            }
            return validation === message;
        }
        else {
            const field = this.fields[fieldName];
            if (field) {
                const validation = get(field, `validations.${code}.${validatorName}`);
                if (!message) {
                    return validation !== undefined;
                }
                return validation === message;
            }
        }
        return false;
    }
    hide(fieldOrDecoratorName, match = true) {
        this.setMetadata(fieldOrDecoratorName, 'visible', !match);
    }
    /**
     *  Register a decorator. Unlike a field, a decorator has no value but can have metadata (like disabled / visible)
     *
     * @param {string} name - the name of the decorator. Must not be in conflict with a field name
     *
     * @return {void}
     */
    initDecorator(name, options = {}) {
        if (!this.decorators[name]) {
            this.decorators[name] = {
                name,
                Displayer: options.Displayer,
            };
            this.defaultMetadata[name] = this.state.metadata[name] || {
                disabled: options.disabled,
                visible: options.visible,
            };
        }
        return this.decorators[name];
    }
    /**
     *  Register a field and return three listeners
     *   - onChange
     *   - onFocus
     *   - onBlur
     *
     * @param {string} name - the name of the field. Must be related to a key in values
     * @param {Validator[]} validators - a object of validators
     * @param {FieldOptions} options
     *
     * @return {Object} a list of listeners for the field and its name
     *                    - name
     *                    - onChange
     *                    - onFocus
     *                    - onBlur
     */
    initField(name, validators = {}, options = {}) {
        const field = this.fields[name];
        if (field === undefined) {
            options.defaultValue = options.defaultValue === undefined ? '' : options.defaultValue;
            options.touchOn = options.touchOn || this.config.touchOn || TouchOn.Blur;
            const isUndefined = options.isUndefined || ((value) => value === undefined);
            // it's possible that validators has been disabled / enabled / added / removed before the insertion of the field in the form
            if (this.placeholderFields[name]) {
                Object.keys(this.placeholderFields[name].validators || {}).forEach((validatorName) => {
                    const placeholderValidator = (this.placeholderFields[name].validators || {})[validatorName];
                    const validator = validators[validatorName];
                    if (validator) {
                        if (typeof placeholderValidator === 'object') {
                            if (typeof validator !== 'object') {
                                validators[validatorName] = {
                                    validator,
                                    disabled: placeholderValidator?.disabled,
                                };
                            }
                            else {
                                validator.disabled = placeholderValidator?.disabled;
                            }
                        }
                    }
                    else if (typeof placeholderValidator === 'object') {
                        if (placeholderValidator.validator !== undefined) {
                            validators[validatorName] = placeholderValidator;
                        }
                    }
                    else {
                        validators[validatorName] = placeholderValidator;
                    }
                });
                delete this.placeholderFields[name].validators;
            }
            this.addField(Object.assign({}, options, simpleMergeDeep({
                name,
                validators,
                validations: [],
                touched: options.touchOn === TouchOn.Load,
                touchOn: options.touchOn,
                context: {
                    name,
                    onChange: (value) => {
                        if (value && value.nativeEvent && value.nativeEvent instanceof Event) {
                            if (!isNaN(value.target.valueAsNumber)) {
                                value = value.target.valueAsNumber;
                            }
                            else {
                                value = value.target.value;
                            }
                        }
                        this.setValue(name, value);
                    },
                    onFocus: () => {
                        const field = this.fields[name];
                        if (field !== undefined && field.touchOn === 'focus' && !field.touched) {
                            this.touch(name);
                        }
                    },
                    onBlur: () => {
                        const field = this.fields[name];
                        if (field !== undefined && field.touchOn === 'blur' && !field.touched) {
                            this.touch(name);
                        }
                    },
                },
            }, this.placeholderFields[name] || {})));
            const initialValue = get(this.state.values, name);
            this.defaultValues[name] = isUndefined(initialValue) ? options.defaultValue : initialValue;
            // we force disable to true if the field is not editable and this is a reconfiguration
            const disabled = (this.config.reconfigure && !options.editable || options.readOnly) ? true : options.disabled;
            this.defaultMetadata[name] = Object.assign({
                disabled,
                visible: options.visible,
                editable: options.editable,
                readOnly: options.readOnly,
            }, this.state.metadata[name]);
        }
        // some options are not dynamically managed by the FormService. We erase the value
        ['Displayer', 'ValueDisplayer', 'label', 'protected', 'containers'].forEach((f) => {
            set(this.fields[name], f, get(options, f));
        });
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this.fields[name].context;
    }
    insert(fieldArrayName, index, initialValue = {}) {
        const currentArrayValue = (get(this.state.values, fieldArrayName) || []);
        if (currentArrayValue.length - 1 >= index) {
            const nextValues = {};
            // need to modifiy all values / metadata / validations with an index superior or equal to the added one
            for (let i = currentArrayValue.length - 1; i >= index; i--) {
                Object.keys(currentArrayValue[i]).forEach((fieldName) => {
                    this.state.validations[`${fieldArrayName}.${i + 1}.${fieldName}`] =
                        this.state.validations[`${fieldArrayName}.${i}.${fieldName}`];
                    this.state.metadata[`${fieldArrayName}.${i + 1}.${fieldName}`] =
                        this.state.metadata[`${fieldArrayName}.${i}.${fieldName}`];
                    if (i === currentArrayValue.length - 1) {
                        const name = `${fieldArrayName}.${i + 1}.${fieldName}`;
                        if (this.fields[`${fieldArrayName}.${i}.${fieldName}`]) {
                            this.addField(Object.assign({}, this.fields[`${fieldArrayName}.${i}.${fieldName}`], {
                                name,
                                context: {
                                    name,
                                    onChange: (value) => {
                                        if (value && value.nativeEvent && value.nativeEvent instanceof Event) {
                                            value = value.target.value;
                                        }
                                        this.setValue(name, value);
                                    },
                                    onFocus: () => {
                                        const field = this.fields[name];
                                        if (field !== undefined && field.touchOn === 'focus' && !field.touched) {
                                            this.touch(name);
                                        }
                                    },
                                    onBlur: () => {
                                        const field = this.fields[name];
                                        if (field !== undefined && field.touchOn === 'blur' && !field.touched) {
                                            this.touch(name);
                                        }
                                    },
                                },
                            }));
                        }
                    }
                    else {
                        const nextField = this.fields[`${fieldArrayName}.${i}.${fieldName}`];
                        const currentField = this.fields[`${fieldArrayName}.${i + 1}.${fieldName}`];
                        if (currentField) {
                            this.fields[`${fieldArrayName}.${i + 1}.${fieldName}`] = Object.assign(nextField, {
                                name: currentField.name,
                                context: currentField.context,
                            });
                        }
                    }
                    delete this.triggered[`${fieldArrayName}.${i}.${fieldName}`];
                    if (i === index) {
                        delete this.state.metadata[`${fieldArrayName}.${i}.${fieldName}`];
                        delete this.state.validations[`${fieldArrayName}.${i}.${fieldName}`];
                        delete this.fields[`${fieldArrayName}.${i}.${fieldName}`];
                    }
                    this.pendingDispatch.validationChange.add(`${fieldArrayName}.${i + 1}.${fieldName}`);
                    this.pendingDispatch.metadataChange.add(`${fieldArrayName}.${i + 1}.${fieldName}`);
                });
            }
            this.pendingDispatch.validationChange.add('');
            nextValues[fieldArrayName] = currentArrayValue.slice(0, index)
                .concat([initialValue])
                .concat(currentArrayValue.slice(index));
            this.setValues(nextValues);
        }
        else {
            this.add(fieldArrayName, initialValue);
        }
    }
    isTouched(fieldName) {
        const field = this.fields[fieldName];
        if (field) {
            return field.touched;
        }
        return false;
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    *loadInitialValues(fetcher) {
        let initialValues;
        let loadingTask = null;
        try {
            loadingTask = yield fork([this, this.delayLoading], this.state.delayLoading || 0);
            if (typeof fetcher === 'string') {
                initialValues = yield asyncGet(fetcher);
            }
            else {
                initialValues = yield fetcher();
            }
            yield cancel(loadingTask);
            this.setInitialValues(initialValues);
            this.setValues(initialValues);
            this.setLoading(false, false);
        }
        catch (e) {
            if (process.env.NODE_ENV === 'development') {
                console.error('Fetch error', e);
            }
            if (loadingTask) {
                yield cancel(loadingTask);
            }
            throw e;
        }
    }
    offChange(type, id) {
        if (this.listenerIndex[id] === undefined)
            return;
        Object.keys(this.listenerIndex[id]).forEach((watch) => {
            this.listeners[type][watch] = this.listeners[type][watch].filter((x) => x.id !== id);
        });
    }
    onChange(type, id, listener, watchs, once) {
        watchs = Array.isArray(watchs) ? watchs : [watchs];
        this.listenerIndex[id] = this.listenerIndex[id] || {};
        for (const watch of watchs) {
            if (this.listenerIndex[id][watch] === undefined) {
                const listenerContext = {
                    id,
                    listener,
                    once,
                    watchs,
                };
                this.listenerIndex[id][watch] = listenerContext;
                this.listeners[type][watch] = this.listeners[type][watch] || [];
                this.listeners[type][watch].push(listenerContext);
                const current = watch === '' ? this.watchIndex[''] : get(this.watchIndex, watch, undefined);
                if (current === undefined) {
                    if (watch === '') {
                        this.watchIndex[''] = true;
                    }
                    else {
                        set(this.watchIndex, watch, true);
                    }
                }
            }
            else {
                // the listener could have been changed (because no useCallback or changes in the dependencies of a useCallback)
                // update the listener via the listenerIndex
                this.listenerIndex[id][watch].listener = listener;
            }
        }
    }
    onMount() {
        if (this.initializing) {
            this.initializing = false;
            this.setMetadatas(this.defaultMetadata);
            this.setValues(this.defaultValues);
        }
    }
    offMetadataChange(id) {
        this.offChange('metadataChange', id);
    }
    onMetadataChange(id, listener, watchs, once = false) {
        this.onChange('metadataChange', id, listener, watchs, once);
    }
    offSubmittingChange(id) {
        this.offChange('submittingChange', id);
    }
    onSubmittingChange(id, listener, once = false) {
        this.onChange('submittingChange', id, listener, '', once);
    }
    offValidationChange(id) {
        this.offChange('validationChange', id);
    }
    onValidationChange(id, listener, watchs, once = false) {
        this.onChange('validationChange', id, listener, watchs, once);
    }
    offValueChange(id) {
        this.offChange('valueChange', id);
    }
    onValueChange(id, listener, watchs, once = false) {
        this.onChange('valueChange', id, listener, watchs, once);
    }
    remove(fieldArrayName, index, changeValue = true) {
        const replace = (currentField, nextField, ignore) => {
            Object.keys(currentField).forEach((key) => {
                if (!ignore.includes(key)) {
                    delete currentField[key];
                }
            });
            Object.keys(nextField).forEach((key) => {
                if (!ignore.includes(key)) {
                    currentField[key] = nextField[key];
                }
            });
        };
        const move = (oldIndex, newIndex) => {
            const nextField = this.fields[oldIndex];
            const currentField = this.fields[newIndex];
            if (nextField) {
                this.state.validations[newIndex] = this.state.validations[oldIndex];
                this.state.metadata[newIndex] = this.state.metadata[oldIndex];
                replace(currentField, nextField, ['name', 'context']);
                this.fields[newIndex] = currentField;
                this.pendingDispatch.validationChange.add(newIndex);
                this.pendingDispatch.metadataChange.add(newIndex);
            }
            else {
                delete this.state.validations[newIndex];
                delete this.state.metadata[newIndex];
                delete this.fields[newIndex];
            }
            delete this.triggered[oldIndex];
        };
        const rm = (oldIndex) => {
            delete this.state.metadata[oldIndex];
            delete this.state.validations[oldIndex];
            delete this.fields[oldIndex];
            delete this.triggered[oldIndex];
        };
        const currentArrayValue = (get(this.state.values, fieldArrayName) || []);
        const last = currentArrayValue.length - 1;
        if (last >= index) {
            const nextValues = {};
            // need to modifiy all values / metadata / validations with an index superior to the removed one
            for (let i = index + 1; i < currentArrayValue.length; i++) {
                if (currentArrayValue[i] === null ||
                    currentArrayValue[i] === undefined ||
                    !(typeof currentArrayValue[i] === 'object')) {
                    // singlecolumntable
                    move(`${fieldArrayName}.${i}`, `${fieldArrayName}.${i - 1}`);
                }
                else {
                    Object.keys(currentArrayValue[i]).forEach((fieldName) => {
                        move(`${fieldArrayName}.${i}.${fieldName}`, `${fieldArrayName}.${i - 1}.${fieldName}`);
                    });
                }
            }
            del(this.fieldIndex, `${fieldArrayName}.${last}`);
            if (currentArrayValue[last] === null ||
                currentArrayValue[last] === undefined ||
                !(typeof currentArrayValue[last] === 'object')) {
                rm(`${fieldArrayName}.${last}`);
            }
            else {
                Object.keys(currentArrayValue[last]).forEach((fieldName) => {
                    rm(`${fieldArrayName}.${last}.${fieldName}`);
                });
            }
            this.pendingDispatch.validationChange.add('');
            nextValues[fieldArrayName] = currentArrayValue.filter((_a, i) => i !== index);
            if (changeValue) {
                this.setValues(nextValues);
            }
        }
    }
    removeValidator(fieldName, validatorName) {
        // get current value
        const field = this.fields[fieldName];
        if (field) {
            delete field.validators[validatorName];
        }
        else if (this.placeholderFields[fieldName] !== undefined &&
            this.placeholderFields[fieldName].validators !== undefined) {
            delete this.placeholderFields[fieldName].validators[validatorName];
        }
        const validations = this.validateAll({
            [fieldName]: this.getValue(fieldName),
        });
        this.compileValidations(Object.keys(validations));
    }
    reset() {
        let props = Object.getOwnPropertyNames(this.fields);
        for (let i = 0; i < props.length; i++) {
            delete this.fields[props[i]];
        }
        props = Object.getOwnPropertyNames(this.decorators);
        for (let i = 0; i < props.length; i++) {
            delete this.decorators[props[i]];
        }
        this.prevValues = {};
        this.prevValidations = {};
        this.prevSubmitting = false;
        this.prevMetadata = {};
        this.triggered = {};
        this.defaultValues = {};
        this.defaultMetadata = {};
        this.listeners = {
            valueChange: {},
            validationChange: {},
            submittingChange: {},
            metadataChange: {},
        };
        this.pendingDispatch = {
            valueChange: new Set(),
            validationChange: new Set(),
            submittingChange: new Set(),
            metadataChange: new Set(),
        };
        this.fieldIndex = {};
        this.watchIndex = {};
        this.listenerIndex = {};
        if (this.state.initialValues === undefined ||
            (this.state.initialValues && typeof this.state.initialValues === 'object')) {
            this.state.values = this.state.initialValues;
        }
        else {
            this.state.values = undefined;
            this.state.fetching = true;
            this.callSaga('loadInitialValues', this.state.initialValues);
        }
        this.state.validations = {};
        this.placeholderFields = {};
        this.initializing = true;
    }
    serializeValidationCode(code) {
        switch (code) {
            case ValidationCode.Loading:
                return ValidationStatus.Loading;
            case ValidationCode.Error:
                return ValidationStatus.Error;
            case ValidationCode.Warning:
                return ValidationStatus.Warning;
            case ValidationCode.Ok:
                return ValidationStatus.Ok;
            default:
                return ValidationStatus.None;
        }
    }
    setContext(key = '', value) {
        if (key === '') {
            this.state.context = value ?? {};
        }
        else {
            set(this.state.context, key, value);
        }
    }
    setError(fieldName, validatorName, message = '', match) {
        return this.setOrClearValidation(ValidationCode.Error, fieldName, validatorName, message, match);
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    setInitialValues(values) {
        this.state.initialValues = values;
    }
    setLoading(loading, fetching) {
        this.state.loading = loading;
        this.state.fetching = fetching;
    }
    setMetadata(fieldOrDecoratorName, key, value, force = true) {
        this.state.metadata[fieldOrDecoratorName] = this.state.metadata[fieldOrDecoratorName] || {};
        if (this.state.metadata[fieldOrDecoratorName][key] === undefined || force) {
            this.state.metadata[fieldOrDecoratorName][key] = value;
            // it's possible that the field is not yet registered
            // we will still try to enable / disable the validator via the placeholderfield
            const exists = Object.keys(this.fields).includes(fieldOrDecoratorName);
            if (key === 'visible' || key === 'disabled') {
                const fieldName = fieldOrDecoratorName;
                if ((key === 'visible' && value === false) || (key === 'disabled' && value === true)) {
                    this.clearError(fieldName, 'required');
                    this.disableValidator(fieldName, 'required');
                }
                else {
                    this.enableValidator(fieldName, 'required');
                    if (exists) {
                        const validations = this.validateAll({
                            [fieldName]: this.getValue(fieldName),
                        });
                        this.compileValidations(Object.keys(validations));
                    }
                }
            }
            this.pendingDispatch.metadataChange.add(fieldOrDecoratorName);
            this.pendingDispatch.metadataChange.add('');
        }
    }
    setMetadatas(metadatas) {
        Object.keys(metadatas).forEach((fieldOrDecoratorName) => {
            Object.keys(metadatas[fieldOrDecoratorName]).forEach((key) => {
                this.setMetadata(fieldOrDecoratorName, key, metadatas[fieldOrDecoratorName][key]);
            });
        });
    }
    setOK(fieldName, validatorName) {
        return this.setOrClearValidation(ValidationCode.Ok, fieldName, validatorName, '', true);
    }
    setOrClearValidation(code, fieldName, validatorName, message = '', match) {
        let changed = false;
        if (fieldName === undefined || this.fields[fieldName]) {
            if (this.hasValidation(fieldName, validatorName, ValidationCode.Loading)) {
                this.clearValidation(fieldName, validatorName, ValidationCode.Loading);
                changed = true;
            }
            if (match === undefined || match) {
                if (!this.hasValidation(fieldName, validatorName, code, message)) {
                    this.setValidation(fieldName, validatorName, code, message);
                    changed = true;
                }
            }
            else if (this.hasValidation(fieldName, validatorName, code)) {
                this.clearValidation(fieldName, validatorName, code);
                changed = true;
            }
        }
        return changed;
    }
    setPendingValidation(fieldName, validatorName, pending = true) {
        return this.setOrClearValidation(ValidationCode.Loading, fieldName, validatorName, '', pending);
    }
    setSubmitting(submitting) {
        this.state.submitting = submitting;
        this.pendingDispatch.submittingChange.add('__submit__');
        this.pendingDispatch.submittingChange.add('');
    }
    setValidation(fieldName, validatorName, code, message = '', compile = true) {
        if (fieldName === undefined) {
            set(this.validations, `${code}.${validatorName}`, message);
        }
        else if (this.fields[fieldName]) {
            set(this.fields[fieldName], `validations.${code}.${validatorName}`, message);
        }
        if (compile) {
            this.compileValidations(fieldName);
        }
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    setValue(fieldName, value) {
        this.setValues({
            [fieldName]: value,
        });
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    setValues(values) {
        const validations = this.validateAll(values);
        Object.keys(values).forEach((key) => {
            const field = this.fields[key];
            if (field && field.touchOn === 'change' && !field.touched) {
                field.touched = true;
            }
            const currentValue = get(this.state.values, key);
            const nextValue = values[key];
            if (Array.isArray(currentValue) && Array.isArray(nextValue) && currentValue.length > nextValue.length) {
                for (let i = nextValue.length; i < currentValue.length; i++) {
                    this.remove(key, i, false);
                }
            }
            set(this.state.values, key, nextValue);
            this._getSubWatchs(key).forEach((key) => this.pendingDispatch.valueChange.add(key));
        });
        this.compileValidations(Object.keys(validations));
    }
    setWarning(fieldName, validatorName, message = '', match) {
        return this.setOrClearValidation(ValidationCode.Warning, fieldName, validatorName, message, match);
    }
    show(fieldOrDecoratorName, match = true) {
        this.setMetadata(fieldOrDecoratorName, 'visible', match);
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    *submit() {
        // compile the validations to get the status
        const { code, fields } = yield this.getContainerFieldValidation(this.state.validations, this.fields, '', false);
        yield this.touchAll();
        if (!this.state.submitting) {
            yield this.setSubmitting(true);
        }
        switch (code) {
            case ValidationCode.Loading:
                this.onValidationChange(generateUniqueId(), () => this.submit(), '', true);
                break;
            case ValidationCode.Error:
                if (typeof this.config.onError === 'function') {
                    yield this.config.onError(fields, this.state.values);
                }
                yield this.setSubmitting(false);
                break;
            case ValidationCode.Warning:
                if (typeof this.config.onWarning === 'function') {
                    yield this.config.onWarning(fields, this.state.values);
                }
                else if (this.state.values) {
                    yield this.config.onSubmit(this.state.values);
                }
                yield this.setSubmitting(false);
                break;
            default:
                if (this.state.values) {
                    yield this.config.onSubmit(this.state.values);
                }
                yield this.setSubmitting(false);
                break;
        }
    }
    touch(fieldName) {
        const field = this.fields[fieldName];
        if (field !== undefined && !field.touched) {
            field.touched = true;
            this.compileValidations(fieldName, true);
        }
    }
    touchAll() {
        Object.keys(this.fields).forEach((fieldName) => {
            this.touch(fieldName);
        });
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    validateSync(fieldName, validatorName, validator, 
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    value) {
        const result = validator(value);
        if (result.valid) {
            this.clearValidation(fieldName, validatorName, ValidationCode.Error, false);
        }
        else {
            this.setValidation(fieldName, validatorName, ValidationCode.Error, result.message, false);
        }
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    *validateAsync(fieldName, validatorName, validator, 
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    value) {
        const result = yield validator(value);
        yield this.clearValidation(fieldName, validatorName, ValidationCode.Loading, false);
        if (result.valid) {
            yield this.clearValidation(fieldName, validatorName, ValidationCode.Error, true);
        }
        else {
            yield this.setValidation(fieldName, validatorName, ValidationCode.Error, result.message, true);
        }
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    validateAll(values) {
        // do all validations
        const tasks = {};
        const validations = {};
        const keys = Object.keys(values);
        for (const key of keys) {
            // we need to find all sub fields (if any) and do the validations for these fields
            const fieldNames = this._getSubFieldNames(key);
            for (const fieldName of fieldNames) {
                const field = this.fields[fieldName];
                if (field !== undefined) {
                    const validators = field.validators;
                    tasks[fieldName] = [];
                    for (const validatorName in validators) {
                        const validator = validators[validatorName];
                        const disabled = typeof validator === 'object' && validator.disabled === true;
                        const async = typeof validator === 'object' && validator.async === true;
                        if (!disabled) {
                            if (async) {
                                this.setValidation(fieldName, validatorName, ValidationCode.Loading, '', false);
                                this.callSaga('validateAsync', fieldName, validatorName, validator.validator, get(values[key], fieldName.slice(key.length + 1)));
                            }
                            else {
                                const validatorSync = typeof validator === 'object'
                                    ? validator.validator
                                    : validator;
                                this.validateSync(fieldName, validatorName, validatorSync, get(values[key], fieldName.slice(key.length + 1)));
                            }
                        }
                    }
                }
            }
        }
        Object.keys(tasks).forEach((fieldName) => {
            validations[fieldName] = this._getValidation(fieldName);
        });
        return validations;
    }
};
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_b = typeof K !== "undefined" && K) === "function" ? _b : Object, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "add", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "addValidator", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "clearError", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Number, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "clearValidation", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "clearWarning", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "compileValidations", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "disable", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "disableValidator", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "enable", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "enableValidator", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "hide", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Number, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "insert", null);
__decorate([
    saga(SagaEffect.Leading)
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    ,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "loadInitialValues", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], FormService.prototype, "onMount", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Number, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "remove", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "removeValidator", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], FormService.prototype, "reset", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Object)
], FormService.prototype, "setContext", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, Object, Boolean]),
    __metadata("design:returntype", Boolean)
], FormService.prototype, "setError", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "setInitialValues", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Boolean, Boolean]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "setLoading", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, typeof (_c = typeof K !== "undefined" && K) === "function" ? _c : Object, Object, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "setMetadata", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "setMetadatas", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Boolean)
], FormService.prototype, "setOK", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, Object, String, Object, Boolean]),
    __metadata("design:returntype", Boolean)
], FormService.prototype, "setOrClearValidation", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, Object]),
    __metadata("design:returntype", Boolean)
], FormService.prototype, "setPendingValidation", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Boolean]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "setSubmitting", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, Number, Object, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "setValidation", null);
__decorate([
    reducer
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    ,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typeof (_d = typeof K !== "undefined" && K) === "function" ? _d : Object, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "setValue", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "setValues", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, Object, Boolean]),
    __metadata("design:returntype", Boolean)
], FormService.prototype, "setWarning", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "show", null);
__decorate([
    saga(SagaEffect.Leading)
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    ,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], FormService.prototype, "submit", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "touch", null);
__decorate([
    reducer,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], FormService.prototype, "touchAll", null);
__decorate([
    saga(SagaEffect.Latest)
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    ,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, Function, Object]),
    __metadata("design:returntype", void 0)
], FormService.prototype, "validateAsync", null);
FormService = __decorate([
    service,
    __metadata("design:paramtypes", [])
], FormService);
export default FormService;
//# sourceMappingURL=FormService.js.map