import { useEffect, useRef } from 'react';
import useForm from './useForm';
import useValue from './useValue';
const defaultIsUndefined = (value) => value === undefined;
const useField = (fieldName, validators = {}, options = {}) => {
    const form = useForm();
    const field = form.initField(fieldName, validators, options);
    const value = useValue(fieldName);
    const isUndefined = options.isUndefined ?? defaultIsUndefined;
    field.value = isUndefined(value) ? (options.defaultValue === undefined ? '' : options.defaultValue) : value;
    const optionsRef = useRef(options);
    const nameRef = useRef(fieldName);
    useEffect(() => {
        if (!form.initializing) {
            field.onChange(field.value);
            const disabled = form.config.reconfigure && !optionsRef.current.editable ? true : optionsRef.current.disabled;
            if (disabled) {
                form.setMetadata(nameRef.current, 'disabled', disabled, false);
            }
            if (optionsRef.current.visible === false) {
                form.setMetadata(nameRef.current, 'visible', optionsRef.current.visible, false);
            }
            form.setMetadata(nameRef.current, 'readOnly', optionsRef.current.readOnly);
            form.setMetadata(nameRef.current, 'editable', optionsRef.current.editable);
        }
    }, [form]);
    return field;
};
export default useField;
//# sourceMappingURL=useField.js.map