import { useCallback, useEffect, useId, useMemo, useState } from 'react';
export function useFormColumnWatcher(form, table, column, listener) {
    const id = useId();
    const [result, setResult] = useState();
    const valueListener = useCallback((value, previousValue, watch) => {
        const listenerResult = listener(value, previousValue, watch);
        if (result !== listenerResult) {
            setResult(listenerResult);
        }
    }, [listener, result]);
    useMemo(() => {
        form.onValueChange(id, valueListener, `${table}.${String(column)}`);
    }, [form, id, table, column, valueListener]);
    useEffect(() => {
        return () => {
            form.offValueChange(id);
        };
    }, [form, id]);
    return result;
}
export default useFormColumnWatcher;
//# sourceMappingURL=useFormColumnWatcher.js.map