import { useCallback, useEffect, useId, useMemo, useRef, useState } from 'react';
// eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
export function useFormWatcher(form, watchs, listener) {
    const id = useId();
    const [result, setResult] = useState();
    const watchRef = useRef(watchs);
    const valueListener = useCallback((value, previousValue, watch) => {
        const listenerResult = listener(value, previousValue, watch);
        if (result !== listenerResult) {
            setResult(listenerResult);
        }
    }, [listener, result]);
    useMemo(() => {
        form.onValueChange(id, valueListener, watchRef.current);
    }, [form, id, valueListener]);
    useEffect(() => {
        return () => {
            form.offValueChange(id);
        };
    }, [form, id]);
    return result;
}
export default useFormWatcher;
//# sourceMappingURL=useFormWatcher.js.map