import { useCallback, useEffect, useId, useState } from 'react';
import useForm from './useForm';
import useFormStatus from './useFormStatus';
const useSubmit = () => {
    const { status, code, fields } = useFormStatus();
    const form = useForm();
    const id = useId();
    const [submitting, setSubmitting] = useState(form.state.submitting || false);
    useEffect(() => {
        const listener = (submitting) => {
            setSubmitting(submitting);
        };
        form.onSubmittingChange(id, listener);
        return () => {
            form.offSubmittingChange(id);
        };
    }, [form, id]);
    const submit = useCallback((e) => {
        if (e) {
            e.preventDefault();
        }
        form.submit();
    }, [form]);
    return {
        submit,
        submitting,
        status,
        code,
        fields,
    };
};
export default useSubmit;
//# sourceMappingURL=useSubmit.js.map