import { useCallback, useEffect, useId, useRef, useState } from 'react';
import { isNullOrEmpty } from '../utils/object';
import { defaultValidation } from './FieldValidation';
import useForm from './useForm';
const useValidation = (name = '', touchedOnly = true) => {
    const form = useForm();
    const id = useId();
    const argsRef = useRef({ name, touchedOnly });
    const getFieldValidation = useCallback((name, touchedOnly) => {
        const field = form.fields[name];
        if (field !== undefined) {
            if (touchedOnly) {
                return field.touched ? form.state.validations[name] || defaultValidation : defaultValidation;
            }
            else {
                return form.state.validations[name] || defaultValidation;
            }
        }
        else {
            return form.getContainerFieldValidation(form.state.validations, form.fields, name, touchedOnly);
        }
    }, [form]);
    const [validation, setValidation] = useState(() => {
        if (isNullOrEmpty(argsRef.current.name)) {
            return form.getContainerFieldValidation(form.state.validations, form.fields, '', argsRef.current.touchedOnly, true);
        }
        else {
            return getFieldValidation(argsRef.current.name, argsRef.current.touchedOnly);
        }
    });
    useEffect(() => {
        const { name, touchedOnly } = argsRef.current;
        const listener = (validation) => {
            if (isNullOrEmpty(name)) {
                setValidation(form.getContainerFieldValidation(form.state.validations, form.fields, '', touchedOnly, true));
            }
            else {
                if (!touchedOnly || form.fields[name]?.touched) {
                    setValidation(validation);
                }
            }
        };
        form.onValidationChange(id, listener, [name]);
        return () => {
            form.offValidationChange(id);
        };
    }, [form, id]);
    return validation;
};
export default useValidation;
//# sourceMappingURL=useValidation.js.map