import { produce } from 'immer';
import { get } from '../utils/object';
import { toLocation } from './utils';
export default class BaseRouter {
    settings = {};
    i18n = {};
    history;
    params = {};
    route;
    constructor() {
        this.history = [];
    }
    get location() {
        return toLocation(window.location.href, this.settings);
    }
    get previousLocation() {
        return this.history[this.history.length - 2];
    }
    get hash() {
        return this.location ? this.location.hash : null;
    }
    get query() {
        return this.location ? this.location.query : null;
    }
    get href() {
        return this.location ? this.location.href : null;
    }
    get pathname() {
        return this.location ? this.location.pathname : null;
    }
    get state() {
        return this.location ? this.location.state : null;
    }
    deleteOrigin() {
        sessionStorage.removeItem('onekijs.from');
    }
    getOrigin() {
        const from = sessionStorage.getItem('onekijs.from') || get(this.settings, 'routes.home', '/');
        return { from };
    }
    init(settings) {
        this.settings = settings;
    }
    saveOrigin(force = true) {
        const currentValue = sessionStorage.getItem('onekijs.from');
        if (!force && currentValue)
            return;
        let from = get(this.settings, 'routes.home', '/');
        const previous = this.history.find((location) => {
            return location.relativeurl && location.relativeurl !== this.history[0].relativeurl;
        });
        if (previous && previous.relativeurl) {
            from = previous.relativeurl;
        }
        sessionStorage.setItem('onekijs.from', from);
    }
    _pushLocation(location, replace = false) {
        this.history = produce(this.history, (draft) => {
            if (replace) {
                draft[0] = location;
            }
            else {
                draft.unshift(location);
                // keep max 20 items
                draft.splice(20, draft.length);
            }
        });
    }
}
//# sourceMappingURL=Router.js.map