import DefaultGlobalService from '../app/GlobalService';
import { SuccessCallback } from '../types/callback';
import { ErrorCallback } from '../types/error';
import { AnonymousObject } from '../types/object';
import { Idp } from './typings';
export default class AuthService extends DefaultGlobalService {
    /**
     * Save the security context in the redux store
     *
     * @param {object} securityContext : the security context to save
     */
    setSecurityContext(securityContext?: AnonymousObject | null, identity?: string): void;
    /**
     * Save the token in the redux store
     *
     * @param {string|object} token : the token to save
     */
    setToken(token: string | AnonymousObject | null, identity?: string, idpName?: string): void;
    /**
     * Save the idp name in the redux store
     *
     * @param {object} idp : the IDP to save (full object). Can be null for removal
     */
    clearIdp(idp: Idp): void;
    /**
     * Save the idp name in the redux store
     *
     * @param {object} idp : the IDP to save (full object). Can be null for removal
     */
    setIdp(idp: Idp): void;
    /**
     * Clear all authentication data from the redux store
     * and the persistence storage
     *
     * @param {object} action :
     *    - onError: callback for any error
     *    - onSuccess: callback for any success
     */
    clear(onError?: ErrorCallback, onSuccess?: SuccessCallback, identity?: string, idpName?: string): Generator<void, void, unknown>;
    /**
     * Get the security context from the server and save it
     *
     * @param {object} action :
     *    - onError: callback for any error
     *    - onSuccess: callback for any success
     * @param {object} context :
     *    - store: the redux store
     *    - router: the OnekiJS router of the application
     *    - settings: the full settings object passed to the application
     */
    fetchSecurityContext(onError?: ErrorCallback, onSuccess?: SuccessCallback, identity?: string): Generator<string | void | Promise<any> | Generator<void | AnonymousObject<any> | Promise<unknown> | Generator<void | Promise<boolean> | import("redux-saga/effects").ForkEffect<AnonymousObject<any> | undefined>, string | AnonymousObject<any> | null, boolean>, string | AnonymousObject<any> | null | undefined, string | (string & AnonymousObject<any>)>, AnonymousObject<any> | null | undefined, AnonymousObject<any> | null>;
    /**
     * Load the token from the persistence storage to the redux store
     *
     * @param {object} action :
     *    - onError: callback for any error
     *    - onSuccess: callback for any success
     * @param {object} context :
     *    - store: the redux store
     *    - settings: the full settings object passed to the application
     */
    loadToken(onError?: ErrorCallback, onSuccess?: SuccessCallback, identity?: string): Generator<void | AnonymousObject<any> | Promise<unknown> | Generator<void | Promise<boolean> | import("redux-saga/effects").ForkEffect<AnonymousObject<any> | undefined>, string | AnonymousObject<any> | null, boolean>, string | AnonymousObject<any> | null | undefined, string | (string & AnonymousObject<any>)>;
    /**
     * Refresh the token against an IDP
     *
     * @param {object} action :
     *    - token: the current oauth token
     *    - idp: the IDP used to refresh the token
     *    - force: refresh the token even the current one is not expired
     *    - onError: callback for any error
     * @param {object} context :
     *    - store: the redux store
     *    - settings: the full settings object passed to the application
     */
    refreshToken(token: AnonymousObject, idp: Idp, force?: boolean, onError?: ErrorCallback): AnonymousObject | undefined;
    /**
     * Validate the token and save it in the store and persistence storage and
     * trigger the refreshing of the token if applicable
     *
     * @param {object} action :
     *    - token: the current oauth token
     *    - idp: the IDP used to refresh the token
     *    - onError: callback for any error
     */
    saveToken(token: AnonymousObject | string, idp: Idp, onError?: ErrorCallback): Generator<void | Promise<boolean> | import("redux-saga/effects").ForkEffect<AnonymousObject<any> | undefined>, string | AnonymousObject<any> | null, boolean>;
}
