import DefaultLocalService from '../app/LocalService';
import DefaultBasicError from '../core/BasicError';
import NotificationService from '../notification/NotificationService';
import { OidcToken } from '../types/auth';
import { SuccessCallback } from '../types/callback';
import { ErrorCallback } from '../types/error';
import { AnonymousObject } from '../types/object';
import AuthService from './AuthService';
import { LoginState } from './typings';
export default class LoginService extends DefaultLocalService<LoginState> {
    notificationService: NotificationService;
    authService: AuthService;
    constructor(notificationService: NotificationService, authService: AuthService);
    /**
     * Inform the user if there is a loading task
     *
     * @param {boolean} loading
     */
    setLoading(loading: boolean): void;
    /**
     * Inform the user if there is an error
     *
     * @param {object} error
     */
    onError(error: DefaultBasicError): void;
    /**
     * Reset the loading and error message after a successful operation
     *
     */
    onSuccess(): void;
    /**
     * Submit the login form
     *
     * @param action:
     *    - username: the value from the username field
     *    - password: the value from the password field
     *    - rememberMe: the value from the rememberMe checkbox
     *    - idpName: name of the IDP (as found in the settings)
     *    - onError: callback for catching possible errors
     * @param context:
     *    - router: an OnekiJS router
     *    - settings: the full settings object passed to the application
     */
    formLogin(data: AnonymousObject, idpName?: string, onError?: ErrorCallback, onSuccess?: SuccessCallback): Generator<unknown, void, [any, any]>;
    /**
     * Redirect the user to an external login page
     *
     * @param action:
     *    - idpName: name of the IDP (as found in the settings)
     *    - onError: callback for catching possible errors
     * @param context:
     *    - store: the Redux store
     *    - router: an OnekiJS router
     *    - settings: the full settings object passed to the application
     */
    externalLogin(idpName?: string, onError?: ErrorCallback): Generator<string | void | Generator<void, void, unknown> | Promise<string>, void, string>;
    /**
     * Parse the token and the security context from the response of the
     * external login
     *
     * @param action:
     *    - idpName: name of the IDP (as found in the settings)
     *    - onError: callback for catching possible errors
     * @param context:
     *    - store: the Redux store
     *    - router: an OnekiJS router
     *    - settings: the full settings object passed to the application
     */
    externalLoginCallback(idpName?: string, onError?: ErrorCallback, onSuccess?: SuccessCallback): Generator<string | void | Promise<any> | [any?, (AnonymousObject<any> | undefined)?] | Generator<void | Generator<void | Promise<boolean> | import("redux-saga/effects").ForkEffect<AnonymousObject<any> | undefined>, string | AnonymousObject<any> | null, boolean> | Generator<string | void | Promise<any> | Generator<void | AnonymousObject<any> | Promise<unknown> | Generator<void | Promise<boolean> | import("redux-saga/effects").ForkEffect<AnonymousObject<any> | undefined>, string | AnonymousObject<any> | null, boolean>, string | AnonymousObject<any> | null | undefined, string | (string & AnonymousObject<any>)>, AnonymousObject<any> | null | undefined, AnonymousObject<any> | null> | import("redux-saga/effects").CallEffect<void>, void, unknown>, void, string & [unknown, any]>;
    /**
     * Save the token and the security context
     *
     * @param action:
     *    - idpName: name of the IDP (as found in the settings)
     *    - onError: callback for catching possible errors
     * @param context:
     *    - router: an OnekiJS router
     *    - settings: the full settings object passed to the application
     */
    successLogin(token?: string | AnonymousObject, securityContext?: AnonymousObject, idpName?: string, onError?: ErrorCallback, onSuccess?: SuccessCallback): Generator<void | Generator<void | Promise<boolean> | import("redux-saga/effects").ForkEffect<AnonymousObject<any> | undefined>, string | AnonymousObject<any> | null, boolean> | Generator<string | void | Promise<any> | Generator<void | AnonymousObject<any> | Promise<unknown> | Generator<void | Promise<boolean> | import("redux-saga/effects").ForkEffect<AnonymousObject<any> | undefined>, string | AnonymousObject<any> | null, boolean>, string | AnonymousObject<any> | null | undefined, string | (string & AnonymousObject<any>)>, AnonymousObject<any> | null | undefined, AnonymousObject<any> | null> | import("redux-saga/effects").CallEffect<void>, void, unknown>;
    /**
     * Check if a login is necessary.
     *
     * @param action:
     *    - idpName: name of the IDP (as found in the settings)
     *    - onError: callback for catching possible errors
     * @param context:
     *    - router: an OnekiJS router
     *    - settings: the full settings object passed to the application
     */
    login(idpName?: string, onError?: ErrorCallback, onSuccess?: SuccessCallback): Generator<void | Generator<void | AnonymousObject<any> | Promise<unknown> | Generator<void | Promise<boolean> | import("redux-saga/effects").ForkEffect<AnonymousObject<any> | undefined>, string | AnonymousObject<any> | null, boolean>, string | AnonymousObject<any> | null | undefined, string | (string & AnonymousObject<any>)> | Generator<string | void | Promise<any> | Generator<void | AnonymousObject<any> | Promise<unknown> | Generator<void | Promise<boolean> | import("redux-saga/effects").ForkEffect<AnonymousObject<any> | undefined>, string | AnonymousObject<any> | null, boolean>, string | AnonymousObject<any> | null | undefined, string | (string & AnonymousObject<any>)>, AnonymousObject<any> | null | undefined, AnonymousObject<any> | null> | Generator<void | Generator<void | Promise<boolean> | import("redux-saga/effects").ForkEffect<AnonymousObject<any> | undefined>, string | AnonymousObject<any> | null, boolean> | Generator<string | void | Promise<any> | Generator<void | AnonymousObject<any> | Promise<unknown> | Generator<void | Promise<boolean> | import("redux-saga/effects").ForkEffect<AnonymousObject<any> | undefined>, string | AnonymousObject<any> | null, boolean>, string | AnonymousObject<any> | null | undefined, string | (string & AnonymousObject<any>)>, AnonymousObject<any> | null | undefined, AnonymousObject<any> | null> | import("redux-saga/effects").CallEffect<void>, void, unknown> | Generator<string | void | Generator<void, void, unknown> | Promise<string>, void, string>, void, OidcToken & AnonymousObject<any>>;
}
