import { CollectionState } from '..';
import { Primitive } from '../types/core';
import { AnonymousObject } from '../types/object';
import { Collection, CollectionBroker, Item, Query, QueryFilter, QueryFilterCriteria, QueryFilterCriteriaOperator, QueryFilterCriteriaValue, QueryFilterId, QueryFilterOrCriteria, QuerySortBy, QuerySortDir, UseCollectionOptions } from './typings';
export default class DefaultCollectionBroker<T = any, I extends Item<T> = Item<T>, S extends CollectionState<T, I> = CollectionState<T, I>, C extends Collection<T, I, S> = Collection<T, I, S>> implements CollectionBroker<T, I, S> {
    protected subscribers: AnonymousObject<C>;
    protected filters: QueryFilter | QueryFilterCriteria | QueryFilterOrCriteria[] | undefined;
    protected subscriberFilters: AnonymousObject<QueryFilter | QueryFilterCriteria | QueryFilterOrCriteria[] | undefined>;
    protected sortBys: string | QuerySortBy | QuerySortBy[] | undefined;
    protected subscriberSortBys: AnonymousObject<string | QuerySortBy | QuerySortBy[] | undefined>;
    protected fields: string[] | undefined;
    protected subscriberFields: AnonymousObject<string[] | undefined>;
    protected params: AnonymousObject | undefined;
    protected subscriberParams: AnonymousObject<AnonymousObject | undefined>;
    protected currentSearch: Primitive | undefined;
    protected subscriberSearch: AnonymousObject<Primitive | undefined>;
    protected currentSort: QuerySortDir | undefined;
    protected subscriberSort: AnonymousObject<QuerySortDir | undefined>;
    protected data: T[] | undefined;
    protected subscriberData: AnonymousObject<T[] | undefined>;
    protected url: string | undefined;
    protected subscriberUrl: AnonymousObject<string | undefined>;
    protected initialLimit: number | undefined;
    protected initialOffset: number | undefined;
    constructor(dataSource: T[] | string | undefined, options: UseCollectionOptions<T, I>);
    addFilter(filterOrCriteria: QueryFilterOrCriteria, parentFilterId?: QueryFilterId, subscriberId?: string): void;
    protected _addFilter(filterOrCriteria: QueryFilterOrCriteria, parentFilterId?: QueryFilterId, subscriberId?: string): void;
    addFilterCriteria(field: string, operator: QueryFilterCriteriaOperator, value: string | number | boolean | QueryFilterCriteriaValue[] | null, not?: boolean | undefined, id?: string | number | symbol | undefined, parentFilterId?: string | number | symbol | undefined, subscriberId?: string): void;
    addSortBy(sortBy: QuerySortBy, prepend?: boolean, subscriberId?: string): void;
    protected _addSortBy(sortBy: QuerySortBy, prepend?: boolean, subscriberId?: string): void;
    addSubscriber(subscriberId: string, subscriber: C): void;
    clearFields(subscriberId?: string): void;
    clearFilter(subscriberId?: string): void;
    clearParam(key: string, subscriberId?: string): void;
    clearParams(subscriberId?: string): void;
    clearSearch(subscriberId?: string): void;
    clearSort(subscriberId?: string): void;
    clearSortBy(subscriberId?: string): void;
    filter(filter: QueryFilter | QueryFilterCriteria | QueryFilterOrCriteria[], subscriberId?: string): void;
    getData(subscriberId?: string): T[] | undefined;
    getFields(subscriberId?: string): string[] | undefined;
    getFilter(subscriberId?: string): QueryFilter | QueryFilterCriteria | QueryFilterOrCriteria[] | undefined;
    getInitialQuery(subscriberId?: string): Query;
    getInitialDataSource(subscriberId?: string): T[] | string | undefined;
    getParams(subscriberId?: string): AnonymousObject | undefined;
    getSearch(subscriberId?: string): Primitive | undefined;
    getSort(subscriberId?: string): QuerySortDir | undefined;
    getSortBy(subscriberId?: string): string | QuerySortBy | QuerySortBy[] | undefined;
    getUrl(subscriberId?: string): string | undefined;
    refresh(query?: Query, subscriberId?: string): void;
    removeFilter(id: QueryFilterId, subscriberId?: string): void;
    protected _removeFilter(id: QueryFilterId, subscriberId?: string): void;
    removeSortBy(id: string, subscriberId?: string): void;
    protected _removeSortBy(id: string, subscriberId?: string): void;
    removeSubscriber(id: string): void;
    search(search: Primitive, subscriberId?: string): void;
    setData(data: T[], query?: Query, subscriberId?: string): void;
    setFields(fields: string[], subscriberId?: string): void;
    setParam(key: string, value: any, subscriberId?: string): void;
    setParams(params: AnonymousObject, subscriberId?: string): void;
    setUrl(url: string, query?: Query, subscriberId?: string): void;
    sort(dir: QuerySortDir, subscriberId?: string): void;
    sortBy(sortBy: string | QuerySortBy | QuerySortBy[], subscriberId?: string): void;
    protected _getSubscribers(subscriberId?: string): C[];
    protected _getQuery(subscriberId?: string): Query;
    protected _setQuery(query: Query, subscriberId?: string): void;
}
