import { Task } from '@redux-saga/types';
import DefaultService from '../core/Service';
import { Primitive } from '../types/core';
import { AnonymousObject } from '../types/object';
import { Location, UnregisterCallback } from '../types/router';
import { Collection, CollectionBy, CollectionFetcherResult, CollectionState, CollectionStatus, Item, LoadingItemStatus, LoadingStatus, LocalQuery, Query, QueryFilter, QueryFilterCriteria, QueryFilterCriteriaOperator, QueryFilterCriteriaValue, QueryFilterId, QueryFilterOrCriteria, QuerySearcher, QuerySerializerResult, QuerySortBy, QuerySortByField, QuerySortComparator, QuerySortDir } from './typings';
export default class CollectionService<T = any, I extends Item<T> = Item<T>, S extends CollectionState<T, I> = CollectionState<T, I>> extends DefaultService<S> implements Collection<T, I, S> {
    _initialState: S;
    _cache: AnonymousObject<any>;
    _idIndex: AnonymousObject<I>;
    _uidIndex: AnonymousObject<I>;
    _positionIndex: AnonymousObject<string>;
    _db?: I[];
    _unregisterRouterListener?: UnregisterCallback;
    _currentQuery?: string;
    _initialQuery?: Query;
    _refreshing?: boolean;
    _noCache?: boolean;
    _noLoading?: boolean;
    _resetData: boolean;
    protected refreshTask?: Task;
    protected followTask?: Task;
    scrollToIndex?: (index: number, options?: {
        align: 'start' | 'center' | 'end' | 'auto';
    }) => void;
    scrollToOffset?: (offsetInPixels: number, options?: {
        align: 'start' | 'center' | 'end' | 'auto';
    }) => void;
    init(): void;
    destroy(): void;
    initDb(dataSource: T[] | string | undefined): void;
    adapt(data: T | null | undefined): I;
    addActive<B extends keyof CollectionBy<T, I>>(by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    addDisabled<B extends keyof CollectionBy<T, I>>(by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    addFilter(filterOrCriteria: QueryFilterOrCriteria, parentFilterId?: QueryFilterId): void;
    addFilterCriteria(field: string, operator: QueryFilterCriteriaOperator, value: string | number | boolean | QueryFilterCriteriaValue[] | null, not?: boolean | undefined, id?: string | number | symbol | undefined, parentFilterId?: string | number | symbol | undefined): void;
    addHighlighted<B extends keyof CollectionBy<T, I>>(by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    addSelected<B extends keyof CollectionBy<T, I>>(by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    addSortBy(sortBy: QuerySortBy, prepend?: boolean): void;
    asService(): this;
    clearFields(): void;
    clearFilter(): void;
    clearLimit(): void;
    clearOffset(): void;
    clearParam(key: string): void;
    clearParams(): void;
    clearSearch(): void;
    clearSort(): void;
    clearSortBy(): void;
    filter(filter: QueryFilter | QueryFilterCriteria | QueryFilterOrCriteria[]): void;
    get data(): T[] | undefined;
    get dataSource(): T[] | string | undefined;
    get hasMore(): boolean;
    get items(): (I | undefined)[];
    get index(): AnonymousObject<I>;
    get status(): CollectionStatus;
    get total(): number | undefined;
    get url(): string;
    getDb(): I[];
    getFields(): string[] | undefined;
    getFilter(): QueryFilter | undefined;
    getFilterById(id: QueryFilterId): QueryFilterOrCriteria | undefined;
    getItem(uid: string): I | undefined;
    getOffset(): number | undefined;
    getParam(key: string): any | undefined;
    getParams(): AnonymousObject | undefined;
    getQuery(): Query;
    getSearch(): Primitive | undefined;
    getLimit(): number | undefined;
    getSort(): QuerySortDir | undefined;
    getSortBy(): QuerySortBy[];
    getSortByField(field: string): QuerySortByField | undefined;
    getSortById(id: string): QuerySortBy | undefined;
    isFiltered(): boolean;
    load(limit?: number, offset?: number, replace?: boolean): void;
    onSubscribe(initialData: T[] | undefined, initialUrl: string | undefined, initialQuery: Query): void;
    query(query: Query): void;
    reload(): void;
    refresh(query?: Query, push?: boolean, noCache?: boolean, noLoading?: boolean): void;
    removeActive<B extends keyof CollectionBy<T, I>>(by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    removeDisabled<B extends keyof CollectionBy<T, I>>(by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    removeFilter(filterId: QueryFilterId): void;
    removeHighlighted<B extends keyof CollectionBy<T, I>>(by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    removeSelected<B extends keyof CollectionBy<T, I>>(by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    removeSortBy(id: string): void;
    reset(): void;
    search(search: Primitive): void;
    serializeQuery(query: Query): QuerySerializerResult;
    setActive<B extends keyof CollectionBy<T, I>>(by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    setData(data: T[], query?: Query): void;
    setDisabled<B extends keyof CollectionBy<T, I>>(by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    setFields(fields: string[]): void;
    setHighlighted<B extends keyof CollectionBy<T, I>>(by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    setMeta<B extends keyof CollectionBy<T, I>, K extends keyof I>(by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][], key: K, value: I[K]): I[];
    setParam(key: string, value: any): void;
    setParams(params: AnonymousObject): void;
    setSelected<B extends keyof CollectionBy<T, I>>(by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    setStatus(status: LoadingStatus): void;
    setUrl(url: string, query?: Query): void;
    sort(dir: QuerySortDir): void;
    sortBy(sortBy: string | QuerySortBy | QuerySortBy[]): void;
    startAutoRefresh(interval: number): Generator<Generator<void, void, unknown> | import("redux-saga/effects").ForkEffect<Generator<void | import("redux-saga/effects").CallEffect<true>, void, unknown>>, void, Task<any>>;
    startFollow(interval: number): Generator<void | Generator<void, void, unknown> | import("redux-saga/effects").ForkEffect<Generator<void | import("redux-saga/effects").CallEffect<true>, void, unknown>>, void, Task<any>>;
    stopAutoRefresh(): Generator<void, void, unknown>;
    stopFollow(): Generator<void, void, unknown>;
    _adapt(data: T | null | undefined, context?: AnonymousObject): I;
    _addByMeta<B extends keyof CollectionBy<T, I>>(field: 'active' | 'disabled' | 'highlighted' | 'selected', by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    _addFilter(query: Query, filterOrCriteria: QueryFilterOrCriteria, parentFilterId?: QueryFilterId): void;
    _addSortBy(query: Query, sortBy: QuerySortBy, prepend?: boolean): void;
    protected _autoRefresh(interval: number): Generator<void | import("redux-saga/effects").CallEffect<true>, void, unknown>;
    _clearFields(query: Query): void;
    _clearFilter(query: Query): void;
    _clearOffset(query: Query): void;
    _clearParam(query: Query, key: string): void;
    _clearParams(query: Query): void;
    _clearSearch(query: Query): void;
    _clearLimit(query: Query): void;
    _clearSortBy(query: Query): void;
    _clearSort(query: Query): void;
    _delayLoading(delay_ms: number, limit?: number | string, offset?: number | string, resetData?: boolean): Generator<void | import("redux-saga/effects").CallEffect<true>, void, unknown>;
    _disableDelayLoading(): void;
    _fetchOnce(): Generator<void | Generator<void | import("redux-saga/effects").ForkEffect<Generator<void | import("redux-saga/effects").CallEffect<true>, void, unknown>> | CollectionFetcherResult<T> | Promise<CollectionFetcherResult<T> | undefined> | import("redux-saga/effects").CancelEffect, CollectionFetcherResult<T>, (Task<any> | null) & CollectionFetcherResult<T> & (CollectionFetcherResult<T> | undefined)>, void, CollectionFetcherResult<T>>;
    _fetch(query: Query, resetData: boolean): Generator<void | Generator<void | import("redux-saga/effects").ForkEffect<Generator<void | import("redux-saga/effects").CallEffect<true>, void, unknown>> | CollectionFetcherResult<T> | Promise<CollectionFetcherResult<T> | undefined> | import("redux-saga/effects").CancelEffect, CollectionFetcherResult<T>, (Task<any> | null) & CollectionFetcherResult<T> & (CollectionFetcherResult<T> | undefined)>, void, CollectionFetcherResult<T>>;
    protected _follow(interval: number): Generator<void | import("redux-saga/effects").CallEffect<true>, void, unknown>;
    _executeFetch(query: Query, options: S["fetchOptions"] | undefined, resetData: boolean): Generator<void | import("redux-saga/effects").ForkEffect<Generator<void | import("redux-saga/effects").CallEffect<true>, void, unknown>> | CollectionFetcherResult<T> | Promise<CollectionFetcherResult<T> | undefined> | import("redux-saga/effects").CancelEffect, CollectionFetcherResult<T>, (Task<any> | null) & CollectionFetcherResult<T> & (CollectionFetcherResult<T> | undefined)>;
    _fetchError(e: any, query: Query): void;
    _fetchSuccess(result: CollectionFetcherResult<T>, resetData: boolean, query: Query): void;
    _buildItem(data: T | null | undefined, adaptee: unknown, _context?: AnonymousObject): I;
    _execute(items: I[], query: LocalQuery, comparator: QuerySortComparator<T>, comparators: AnonymousObject<QuerySortComparator<T>>, searcher?: QuerySearcher<T>): I[];
    _getId(data: T): string | number | undefined;
    _indexDb(item: I, _context?: AnonymousObject): void;
    _indexId(item: I, _context?: AnonymousObject): void;
    _indexItem(item: I, context?: AnonymousObject): void;
    _indexPosition(item: I, context?: AnonymousObject): void;
    _indexUid(item: I, _context?: AnonymousObject): void;
    _isFiltered(query: Query): boolean;
    _onLocationChange(location: Location): void;
    _parseLocation(location: Location): Query;
    _removeByMeta<B extends keyof CollectionBy<T, I>>(field: 'active' | 'disabled' | 'highlighted' | 'selected', by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    _removeFilter(query: Query, filterId: QueryFilterId): void;
    _removeSortBy(query: Query, sort: QuerySortBy): void;
    _removeSortById(query: Query, id: string): void;
    _setByMeta<B extends keyof CollectionBy<T, I>>(field: 'active' | 'disabled' | 'highlighted' | 'selected', by: B, target: CollectionBy<T, I>[B] | CollectionBy<T, I>[B][]): I[];
    _setFields(query: Query, fields: string[]): void;
    _setFilter(query: Query, filter: QueryFilter | QueryFilterCriteria | QueryFilterOrCriteria[]): void;
    _setItems(items: (I | undefined)[]): void;
    _setLimit(query: Query, limit: number): void;
    _setLimitOffset(query: Query, limit?: number, offset?: number): void;
    _setLoading(options?: {
        status?: LoadingItemStatus;
        limit?: number;
        offset?: number;
        resetLimit?: boolean;
        resetData?: boolean;
    }): void;
    _setQuery(query: Query, force?: boolean): void;
    _setOffset(query: Query, offset: number): void;
    _setParam(query: Query, key: string, value: any): void;
    _setParams(query: Query, params: AnonymousObject): void;
    _setSearch(query: Query, search: Primitive): void;
    _setSort(query: Query, dir: QuerySortDir): void;
    _setSortBy(query: Query, sortBy: string | QuerySortBy | QuerySortBy[]): void;
}
