import { Primitive } from '../types/core';
import { AnonymousObject } from '../types/object';
import { Collection, CollectionState, CollectionStatus, Item, LocalQuery, Query, QueryFilter, QueryFilterCriteria, QueryFilterCriteriaOperator, QueryFilterCriteriaValue, QueryFilterId, QueryFilterOrCriteria, QuerySearcher, QuerySerializer, QuerySortBy, QuerySortByField, QuerySortByMultiFields, QuerySortComparator, QuerySortDir } from './typings';
export declare const rootFilterId: unique symbol;
export declare const applyCriteria: <T = any, I extends Item<T> = Item<T>>(item: I, criteria: QueryFilterCriteria) => boolean;
export declare const applyFields: <T = any, I extends Item<T> = Item<T>>(items: I[], fields?: string[]) => I[];
export declare const applyFilter: <T = any, I extends Item<T> = Item<T>>(item: I, filter?: QueryFilter) => boolean;
export declare const applyOperator: (operator: QueryFilterCriteriaOperator, left: any, right?: QueryFilterCriteriaValue | QueryFilterCriteriaValue[]) => boolean;
export declare const applySearch: <T = any, I extends Item<T> = Item<T>>(item: I, search?: QueryFilterCriteriaValue, searcher?: QuerySearcher<T> | undefined) => boolean;
export declare const applySort: <T = any, I extends Item<T> = Item<T>>(items: I[], dir: QuerySortDir, comparator: QuerySortComparator<T>) => I[];
export declare const applySortBy: <T = any, I extends Item<T> = Item<T>>(items: I[], sortBy: QuerySortBy[], comparators: AnonymousObject<QuerySortComparator<T>>) => I[];
export declare const defaultComparator: (a: any, b: any) => 1 | 0 | -1;
export declare const defaultQueryEngine: <T = any, I extends Item<T> = Item<T>>(items: I[], query: LocalQuery, comparator: QuerySortComparator<T>, comparators: AnonymousObject<QuerySortComparator<T>>, searcher?: QuerySearcher<T> | undefined) => I[];
export declare const defaultSerializer: QuerySerializer;
export declare const urlSerializer: QuerySerializer;
export declare const generateFilterId: () => number;
export declare const isCollectionReady: <T, I extends Item<T>>(collection: Collection<T, I>) => boolean;
export declare const isCollectionLoading: <T, I extends Item<T>>(collection: Collection<T, I>) => boolean;
export declare const isCollectionFetching: <T, I extends Item<T>>(collection: Collection<T, I>) => boolean;
export declare const isCollectionInitializing: <T, I extends Item<T>>(collection: Collection<T, I>) => boolean;
export declare const isItemLoading: <T, I extends Item<T>>(item?: I | undefined) => boolean;
export declare const isItemFetching: <T, I extends Item<T>>(item?: I | undefined) => boolean;
export declare const isQueryFilter: (value: QueryFilterOrCriteria | QueryFilterOrCriteria[]) => value is QueryFilter;
export declare const isQueryFilterString: (filter: string) => boolean;
export declare const getQueryFilter: (filter: string) => QueryFilter | undefined;
export declare const getQueryFilterCriteriaValue: (value: string) => QueryFilterCriteriaValue;
export declare const getQueryFilterCriteria: (filterCriteria: string) => QueryFilterCriteria | undefined;
export declare const getQueryFilterOrCriterias: (filterOrCriterias: string) => QueryFilterOrCriteria[];
export declare const isQueryFilterCriteria: (value: QueryFilterOrCriteria) => value is QueryFilterCriteria;
export declare const isQuerySortByField: (value: QuerySortBy) => value is QuerySortByField;
export declare const isQuerySortByMultiFields: (value: QuerySortBy) => value is QuerySortByMultiFields;
export declare const isSameFilterCriteriaValue: (v1: QueryFilterCriteriaValue | QueryFilterCriteriaValue[], v2: QueryFilterCriteriaValue | QueryFilterCriteriaValue[]) => boolean;
export declare const isSameFilter: (f1?: QueryFilterOrCriteria, f2?: QueryFilterOrCriteria) => boolean;
export declare const isSameQuery: (q1: Query, q2: Query) => boolean;
export declare const isSameSortBy: (s1?: QuerySortBy | QuerySortBy[], s2?: QuerySortBy | QuerySortBy[]) => boolean;
export declare const parseQuery: (query: URLSearchParams | AnonymousObject) => Query;
export declare const shouldResetData: (query: Query, nextQuery: Query) => boolean;
export declare const parseQueryFilter: (filter: string) => QueryFilter;
export declare const parseSortBy: (sortBy: string) => QuerySortBy[];
export declare const serializeCriteria: (criteria: QueryFilterCriteria) => string;
export declare const serializeFields: (fields: string[] | undefined) => string | void;
export declare const serializeParams: (params: AnonymousObject | undefined) => AnonymousObject<string> | void;
export declare const serializeSearch: (search: Primitive | undefined) => string | void;
export declare const serializeFilter: (filter: QueryFilter | undefined) => string | void;
export declare const serializeOffset: (offset: number | undefined) => string | void;
export declare const serializeLimit: (limit: number | undefined) => string | void;
export declare const serializeSort: (sort: QuerySortDir | undefined) => string | void;
export declare const serializeSortBy: (sortBy: QuerySortBy[] | undefined) => string | void;
export declare const serializeSubFilter: (filter: QueryFilter) => string;
export declare const serializeValue: (value: QueryFilterCriteriaValue | QueryFilterCriteriaValue[]) => string;
export declare const serializePrimitiveValue: (value: QueryFilterCriteriaValue) => string;
export declare const visitFilter: (filter: QueryFilter, visitor: (filter: QueryFilter) => boolean | void) => boolean | void;
export declare const addFilter: (query: Query, filterOrCriteria: QueryFilterOrCriteria, parentFilterId?: QueryFilterId) => void;
export declare const isCollection: <T, I extends Item<T>, S extends CollectionState<T, I>>(data?: string | Collection<T, I, S> | T[] | undefined) => data is Collection<T, I, S>;
export declare const dummyLogMetadata: () => void;
export declare const formatFilter: (filter?: QueryFilter | QueryFilterCriteria | QueryFilterOrCriteria[]) => QueryFilter | undefined;
export declare const toQuerySortBy: (sortBy: QuerySortBy | QuerySortBy[] | string | undefined) => QuerySortBy[];
export declare const formatSortBy: (sortBy: string | QuerySortBy | QuerySortBy[] | undefined, currentSortBy?: QuerySortBy[]) => QuerySortBy[];
export declare const getDefaultCollectionStatus: (dataOrUrl: any[] | string | undefined, brokerable?: boolean, brokered?: boolean, fetchOnce?: boolean) => CollectionStatus;
