import { MutableRefObject } from 'react';
import { AppSettings } from '../types/app';
import { I18n } from '../types/i18n';
import { AnonymousObject } from '../types/object';
import { LinkProps, Location, LocationChangeCallback, ParsedQuery, Router, RouterPushOptions, UnregisterCallback } from '../types/router';
export default abstract class BaseRouter implements Router {
    settings: AppSettings;
    i18n: I18n;
    history: Location[];
    params: AnonymousObject;
    route?: string;
    constructor();
    get location(): Location;
    get previousLocation(): Location;
    get hash(): ParsedQuery<string> | null | undefined;
    get query(): ParsedQuery<string> | null | undefined;
    get href(): string | null | undefined;
    get pathname(): string | null;
    get state(): AnonymousObject | null | undefined;
    abstract back(delta?: number): void;
    deleteOrigin(): void;
    abstract forward(delta?: number): void;
    abstract getLinkComponent(props: LinkProps, ref: ((instance: HTMLAnchorElement | null) => void) | MutableRefObject<HTMLAnchorElement | null> | null): JSX.Element;
    getOrigin(): {
        from: string;
    };
    init(settings: AppSettings): void;
    /**
     * callback(url) where url is:
     * {
     *   url: string, // example: /users/1?test=1&test2#h=3&h2
     *   route: string, // example: /users/[id]
     *   pathname: string, // example: /users/1
     *   query: obj, // example: {test:1,test2:null}
     *   hash: obj // example: {h:3, h2:null}
     *   state: obj // example: {key1: 'value1'}
     * }
     */
    abstract listen(callback: LocationChangeCallback): UnregisterCallback;
    /**
     * url can be a string or a location.
     * If location, the format is the following
     * {
     *   url: string, // example: /users/1?test=1&test2#h=3&h2
     *   route: string, // example: /users/[id]
     *   pathname: string, // example: /users/1
     *   query: obj, // example: {test:1,test2:null}
     *   hash: obj // example: {h:3, h2:null}
     *   state: obj // example: {key1: 'value1'}
     * }
     */
    abstract push(urlOrLocation: string | Location, options?: RouterPushOptions): void;
    abstract replace(urlOrLocation: string | Location, options?: RouterPushOptions): void;
    saveOrigin(force?: boolean): void;
    _pushLocation(location: Location, replace?: boolean): void;
}
