/**
 * Add a browser cookie (created on the browser side)
 *
 * @param {string} name : name of the cookie
 * @param {string} value : value of the cookie
 * @param {boolean} crypt : encrypt the cookie (not usable by the server)
 * @param {integer} ttl : time to live of the cookie (in seconds)
 * @param {string} path : scope of the cookie
 */
export declare function addCookie(name: string, value?: string, crypt?: boolean, ttl?: number, path?: string): Promise<void>;
/**
 * Get a cookie by name. Decrypt it if necessary
 * check https://stackoverflow.com/questions/10730362/get-cookie-by-name
 *
 * @param {string} name : name of the cookie
 * @param {boolean} secure : decrypt the cookie
 */
export declare function getCookie(name: string, secure?: boolean): Promise<any>;
/**
 * convert a TTL (time to live) to an expire date
 *
 * @param {int} ttl : time to live in seconds
 */
export declare const getCookieExpireTime: (ttl: number) => string;
/**
 * get a value of a key from the specified storage
 * if the storage is set to any, try all storage
 *
 * @param {string} key : name of the key to get the value from
 * @param {string} storage : localStorage, sessionStorage, cookie, any (defaults to any)
 * @param {boolean} secure : decrypt the cookie (defaults to true)
 */
export declare function getItem(key: string, storage?: string, secure?: boolean): Promise<unknown>;
/**
 * Listen to a change in the localstorage
 *
 * @param {string} key : key of the local storage entry
 * @param {function} listener : a function receiving the old value an the new value
 */
export declare function onStorageChange(key: string, listener: (newValue: any, oldValue: any) => void): void;
/**
 * Remove a browser cookie (created on the browser side)
 *
 * @param {string} name : name of the cookie
 * @param {string} path : scope of the cookie
 */
export declare const removeCookie: (name: string, path?: string) => void;
/**
 * remove a key from the specified storage
 * if the storage is set to 'any', try all storage
 *
 * @param {string} key : name of the key to get the value from
 * @param {string} storage : localStorage, sessionStorage, cookie, any (defaults to any)
 */
export declare function removeItem(key: string, storage?: string | null): void;
/**
 * Add a key/value in the specified storage
 *
 * @param {*} key : key of the item
 * @param {*} value : value of the item
 * @param {*} storage : localStorage, sessionStorage or cookie
 * @param {*} crypt : encrypt the value (only if cookie). Defaults to true
 * @param {*} ttl : time to live of the cookie (in seconds). If null, the cookie is removed when the browser is closed
 * @param {*} path : scope of the cookie. Defaults to /
 */
export declare function setItem(key: string, value: string, storage: string | null | undefined, crypt?: boolean, ttl?: number, path?: string): Promise<void>;
